@@ -32,16 +32,6 @@ namespace mozilla {
 
 using namespace dom;
 
-/******************************************************************************
- * mozilla::EditActionResult
- *****************************************************************************/
-
-EditActionResult& EditActionResult::operator|=(
-    const MoveNodeResult& aMoveNodeResult) {
-  mHandled |= aMoveNodeResult.Handled();
-  return *this;
-}
-
 /******************************************************************************
  * some general purpose editor utils
  *****************************************************************************/
@@ -89,8 +79,8 @@ bool EditorUtils::IsDescendantOf(const nsINode& aNode, const nsINode& aParent,
 }
 
 // static
-Maybe<StyleWhiteSpace> EditorUtils::GetComputedWhiteSpaceStyle(
-    const nsIContent& aContent) {
+Maybe<std::pair<StyleWhiteSpaceCollapse, StyleTextWrapMode>>
+EditorUtils::GetComputedWhiteSpaceStyles(const nsIContent& aContent) {
   if (MOZ_UNLIKELY(!aContent.IsElement() && !aContent.GetParentElement())) {
     return Nothing();
   }
@@ -101,7 +91,9 @@ Maybe<StyleWhiteSpace> EditorUtils::GetComputedWhiteSpaceStyle(
   if (NS_WARN_IF(!elementStyle)) {
     return Nothing();
   }
-  return Some(elementStyle->StyleText()->mWhiteSpace);
+  const auto* styleText = elementStyle->StyleText();
+  return Some(
+      std::pair(styleText->mWhiteSpaceCollapse, styleText->mTextWrapMode));
 }
 
 // static
@@ -164,39 +156,8 @@ bool EditorUtils::IsOnlyNewLinePreformatted(const nsIContent& aContent) {
     return false;
   }
 
-  return elementStyle->StyleText()->mWhiteSpace == StyleWhiteSpace::PreLine;
-}
-
-bool EditorUtils::IsPointInSelection(const Selection& aSelection,
-                                     const nsINode& aParentNode,
-                                     uint32_t aOffset) {
-  if (aSelection.IsCollapsed()) {
-    return false;
-  }
-
-  const uint32_t rangeCount = aSelection.RangeCount();
-  for (const uint32_t i : IntegerRange(rangeCount)) {
-    MOZ_ASSERT(aSelection.RangeCount() == rangeCount);
-    RefPtr<const nsRange> range = aSelection.GetRangeAt(i);
-    if (MOZ_UNLIKELY(NS_WARN_IF(!range))) {
-      // Don't bail yet, iterate through them all
-      continue;
-    }
-
-    IgnoredErrorResult ignoredError;
-    bool nodeIsInSelection =
-        range->IsPointInRange(aParentNode, aOffset, ignoredError) &&
-        !ignoredError.Failed();
-    NS_WARNING_ASSERTION(!ignoredError.Failed(),
-                         "nsRange::IsPointInRange() failed");
-
-    // Done when we find a range that we are in
-    if (nodeIsInSelection) {
-      return true;
-    }
-  }
-
-  return false;
+  return elementStyle->StyleText()->mWhiteSpaceCollapse ==
+         StyleWhiteSpaceCollapse::PreserveBreaks;
 }
 
 // static
@@ -391,6 +352,15 @@ bool EditorDOMPointBase<
          EditorUtils::IsOnlyNewLinePreformatted(*ContainerAs<Text>());
 }
 
+template <typename PT, typename CT>
+bool EditorDOMPointBase<PT, CT>::IsContainerEditableRoot() const {
+  if (MOZ_UNLIKELY(!mParent) || MOZ_UNLIKELY(!mParent->IsEditable()) ||
+      NS_WARN_IF(mParent->IsInNativeAnonymousSubtree())) {
+    return false;
+  }
+  return HTMLEditUtils::ElementIsEditableRoot(*mParent);
+}
+
 /******************************************************************************
  * mozilla::EditorDOMRangeBase
  *****************************************************************************/
