@@ -255,6 +255,7 @@ class PrefRow {
       if (this.editing) {
         this.inputField = document.createElement("input");
         this.inputField.value = this.value;
+        this.inputField.ariaLabel = this.name;
         if (this.type == "Number") {
           this.inputField.type = "number";
           this.inputField.required = true;
@@ -349,7 +350,7 @@ class PrefRow {
       className =
         (this.hasUserValue ? "has-user-value " : "") +
         (this.isLocked ? "locked " : "") +
-        (this.exists ? "" : "deleted ") +
+        (this.exists || this.isAddRow ? "" : "deleted ") +
         (this.isAddRow ? "add " : "") +
         (this.odd ? "odd " : "");
     }
@@ -508,7 +509,7 @@ function loadPrefs() {
     });
   });
 
-  showAll.addEventListener("click", event => {
+  showAll.addEventListener("click", () => {
     search.focus();
     search.value = "";
     gFilterPrefsTask.disarm();
