@@ -2,14 +2,12 @@
 /* vim: set sts=2 sw=2 et tw=80: */
 "use strict";
 
-const { Downloads } = ChromeUtils.import(
-  "resource://gre/modules/Downloads.jsm"
+const { Downloads } = ChromeUtils.importESModule(
+  "resource://gre/modules/Downloads.sys.mjs"
 );
 
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
-
-const { TestUtils } = ChromeUtils.import(
-  "resource://testing-common/TestUtils.jsm"
+const { TestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TestUtils.sys.mjs"
 );
 
 const server = createHttpServer();
@@ -54,7 +52,6 @@ function handleRequest(request, response) {
         "Requested Range Not Satisfiable"
       );
       response.setHeader("Content-Range", `*/${TOTAL_LEN}`, false);
-      response.finish();
       return;
     }
 
@@ -78,7 +75,7 @@ function handleRequest(request, response) {
     try {
       response.finish();
     } catch (e) {
-      // This will throw, but we don't care at this point.
+      // This can throw, but we don't care at this point.
     }
   });
 }
@@ -175,7 +172,7 @@ function backgroundScript() {
         // expected events and we're not looking for an exact match,
         // then we just may not have seen the event yet, so return without
         // failing and check() will be called again when a new event arrives.
-        if (expected.length == 0) {
+        if (!expected.length) {
           events = remaining;
           eventWaiter = null;
           resolve();
@@ -232,19 +229,16 @@ let downloadDir;
 let extension;
 
 async function waitForCreatedPartFile(baseFilename = "interruptible.html") {
-  const partFilePath = `${downloadDir.path}/${baseFilename}.part`;
+  const partFilePath = PathUtils.join(downloadDir.path, `${baseFilename}.part`);
 
   info(`Wait for ${partFilePath} to be created`);
   let lastError;
   await TestUtils.waitForCondition(
-    async () =>
-      OS.File.stat(partFilePath).then(
-        () => true,
-        err => {
-          lastError = err;
-          return false;
-        }
-      ),
+    () =>
+      IOUtils.exists(partFilePath).catch(err => {
+        lastError = err;
+        return false;
+      }),
     `Wait for the ${partFilePath} to exists before pausing the download`
   ).catch(err => {
     if (lastError) {
@@ -254,11 +248,16 @@ async function waitForCreatedPartFile(baseFilename = "interruptible.html") {
   });
 }
 
-async function clearDownloads(callback) {
+async function clearDownloads() {
   let list = await Downloads.getList(Downloads.ALL);
   let downloads = await list.getAll();
 
-  await Promise.all(downloads.map(download => list.remove(download)));
+  await Promise.all(
+    downloads.map(async download => {
+      await download.finalize(true);
+      list.remove(download);
+    })
+  );
 
   return downloads;
 }
@@ -288,7 +287,7 @@ async function waitForProgress(url, testFn) {
   });
 }
 
-add_task(async function setup() {
+add_setup(async () => {
   const nsIFile = Ci.nsIFile;
   downloadDir = FileUtils.getDir("TmpD", ["downloads"]);
   downloadDir.createUnique(nsIFile.DIRECTORY_TYPE, FileUtils.PERMS_DIRECTORY);
@@ -297,12 +296,11 @@ add_task(async function setup() {
   Services.prefs.setIntPref("browser.download.folderList", 2);
   Services.prefs.setComplexValue("browser.download.dir", nsIFile, downloadDir);
 
-  registerCleanupFunction(() => {
+  registerCleanupFunction(async () => {
     Services.prefs.clearUserPref("browser.download.folderList");
     Services.prefs.clearUserPref("browser.download.dir");
+    await clearDownloads();
     downloadDir.remove(true);
-
-    return clearDownloads();
   });
 
   await clearDownloads().then(downloads => {
@@ -318,6 +316,10 @@ add_task(async function setup() {
 
   await extension.startup();
   await extension.awaitMessage("ready");
+
+  registerCleanupFunction(async () => {
+    await extension.unload();
+  });
 });
 
 add_task(async function test_events() {
@@ -411,7 +413,7 @@ add_task(async function test_cancel() {
   equal(msg.result[0].paused, false, "download.paused is correct");
   equal(
     msg.result[0].estimatedEndTime,
-    null,
+    undefined,
     "download.estimatedEndTime is correct"
   );
   equal(msg.result[0].canResume, false, "download.canResume is correct");
@@ -495,7 +497,7 @@ add_task(async function test_pauseresume() {
   equal(msg.result[0].paused, true, "download.paused is correct");
   equal(
     msg.result[0].estimatedEndTime,
-    null,
+    undefined,
     "download.estimatedEndTime is correct"
   );
   equal(msg.result[0].canResume, true, "download.canResume is correct");
@@ -566,7 +568,7 @@ add_task(async function test_pauseresume() {
   equal(msg.result[0].paused, false, "download.paused is correct");
   equal(
     msg.result[0].estimatedEndTime,
-    null,
+    undefined,
     "download.estimatedEndTime is correct"
   );
   equal(msg.result[0].canResume, false, "download.canResume is correct");
@@ -649,7 +651,7 @@ add_task(async function test_pausecancel() {
   equal(msg.result[0].paused, true, "download.paused is correct");
   equal(
     msg.result[0].estimatedEndTime,
-    null,
+    undefined,
     "download.estimatedEndTime is correct"
   );
   equal(msg.result[0].canResume, true, "download.canResume is correct");
@@ -699,7 +701,7 @@ add_task(async function test_pausecancel() {
   equal(msg.result[0].paused, false, "download.paused is correct");
   equal(
     msg.result[0].estimatedEndTime,
-    null,
+    undefined,
     "download.estimatedEndTime is correct"
   );
   equal(msg.result[0].canResume, false, "download.canResume is correct");
@@ -758,18 +760,119 @@ add_task(async function test_file_removal() {
     "error",
     "removeFile() fails since the file was already removed."
   );
-  ok(
-    /file doesn't exist/.test(msg.errmsg),
+  equal(
+    msg.errmsg,
+    `Could not remove download id ${id} because the file doesn't exist`,
     "removeFile() failed on removed file."
   );
 
   msg = await runInExtension("removeFile", 1000);
-  ok(
-    /Invalid download id/.test(msg.errmsg),
+  equal(
+    msg.errmsg,
+    "Invalid download id 1000",
     "removeFile() failed due to non-existent id"
   );
 });
 
+add_task(async function test_file_removeFile_permission_failure() {
+  const inputDirname = "subdir_for_download";
+  const inputFilename = "downloaded_filename.txt";
+  const expectedDir = PathUtils.join(downloadDir.path, inputDirname);
+  const expectedPath = PathUtils.join(expectedDir, inputFilename);
+
+  let msg = await runInExtension("download", {
+    url: TXT_URL,
+    filename: `${inputDirname}/${inputFilename}`,
+  });
+  equal(msg.status, "success", "download() succeeded");
+  const id = msg.result;
+
+  msg = await runInExtension("waitForEvents", [
+    { type: "onCreated", data: { id, url: TXT_URL } },
+    {
+      type: "onChanged",
+      data: {
+        id,
+        state: {
+          previous: "in_progress",
+          current: "complete",
+        },
+      },
+    },
+  ]);
+
+  equal(msg.status, "success", "got onCreated and onChanged events");
+
+  msg = await runInExtension("search", { id });
+  equal(msg.status, "success", "search() succeeded");
+  equal(expectedPath, msg.result[0]?.filename, "Got expected filename");
+
+  async function withUndeletableFileUnix(testRemoveFile) {
+    try {
+      // Temporarily make directory unreadable/inaccessible.
+      await IOUtils.setPermissions(expectedDir, 0);
+      // Remove should fail with Unix error 13 (EACCES).
+      await testRemoveFile();
+    } finally {
+      await IOUtils.setPermissions(expectedDir, 0o777);
+    }
+  }
+  async function withUndeletableFileWin(testRemoveFile) {
+    // On Windows, a directory marked as read-only does not prevent the deletion
+    // of its content. So we need an alternative approach here.
+    let stream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(
+      Ci.nsIFileInputStream
+    );
+    try {
+      // Open a file handle. The file cannot be deleted until it is closed.
+      stream.init(await IOUtils.getFile(expectedPath), -1, 0, 0);
+      // Remove should fail with Win error 32 (ERROR_SHARING_VIOLATION).
+      await testRemoveFile();
+    } finally {
+      stream.close();
+    }
+  }
+
+  const withUndeletableFile =
+    AppConstants.platform === "win"
+      ? withUndeletableFileWin
+      : withUndeletableFileUnix;
+
+  let consoleOutput;
+  await withUndeletableFile(async () => {
+    consoleOutput = await promiseConsoleOutput(async () => {
+      msg = await runInExtension("removeFile", id);
+    });
+  });
+
+  equal(msg.status, "error", "removeFile() fails due to missing dir perms");
+  // Verify that an unexpected error is redacted, with a useful error message
+  // logged to the console.
+  // Note: if we ever decide to make the error for permission failures more
+  // useful, try to add a new test case for unexpected errors, even if
+  // completely artificial such as mocking + breaking an internal API.
+  equal(msg.errmsg, "An unexpected error occurred", "Error message redacted");
+
+  AddonTestUtils.checkMessages(consoleOutput.messages, {
+    expected: [{ message: /NotAllowedError/ }],
+  });
+
+  ok(await IOUtils.exists(expectedPath), "File exists before removeFile()");
+
+  msg = await runInExtension("removeFile", id);
+  equal(msg.status, "success", "removeFile() succeeded");
+
+  equal(await IOUtils.exists(expectedPath), false, "File was really removed");
+
+  // As a bonus: check that the re-created file can be deleted without issues.
+  await IOUtils.writeUTF8(expectedPath, "content here");
+
+  msg = await runInExtension("removeFile", id);
+  equal(msg.status, "success", "removeFile() succeeded after recreation");
+
+  equal(await IOUtils.exists(expectedPath), false, "File was removed again");
+});
+
 add_task(async function test_removal_of_incomplete_download() {
   const filename = "remove-incomplete.html";
   let url = getInterruptibleUrl(filename);
@@ -937,7 +1040,10 @@ add_task(async function test_erase() {
 
   msg = await runInExtension(
     "waitForEvents",
-    [{ type: "onErased", data: ids.dl2 }, { type: "onErased", data: ids.dl3 }],
+    [
+      { type: "onErased", data: ids.dl2 },
+      { type: "onErased", data: ids.dl3 },
+    ],
     { inorder: false }
   );
   equal(msg.status, "success", "received 2 onErased events");
@@ -959,7 +1065,7 @@ add_task(async function test_getFileIcon() {
   let docShell = webNav.docShell;
 
   let system = Services.scriptSecurityManager.getSystemPrincipal();
-  docShell.createAboutBlankContentViewer(system);
+  docShell.createAboutBlankDocumentViewer(system, system);
 
   let img = webNav.document.createElement("img");
 
@@ -968,6 +1074,14 @@ add_task(async function test_getFileIcon() {
   const id = msg.result;
 
   msg = await runInExtension("getFileIcon", id);
+  if (AppConstants.platform === "linux" && Services.env.get("MOZ_HEADLESS")) {
+    // Linux doesn't have moz-icon: support when headless. It relies on GTK.
+    // See bug 1955705 to try to lift this restriction.
+    equal(msg.status, "error", "getFileIcon() failed");
+    webNav.close();
+    return;
+  }
+
   equal(msg.status, "success", "getFileIcon() succeeded");
   await loadImage(img, msg.result);
   equal(img.height, 32, "returns an icon with the right height");
@@ -1032,7 +1146,11 @@ add_task(async function test_estimatedendtime() {
   equal(msg.status, "success", "search() succeeded");
   equal(msg.result.length, 1, "search() found 1 download");
   ok(msg.result[0].estimatedEndTime, "download.estimatedEndTime is correct");
-  ok(msg.result[0].bytesReceived > 0, "download.bytesReceived is correct");
+  Assert.greater(
+    msg.result[0].bytesReceived,
+    0,
+    "download.bytesReceived is correct"
+  );
 
   msg = await runInExtension("cancel", id);
 
@@ -1042,6 +1160,21 @@ add_task(async function test_estimatedendtime() {
   ok(!msg.result[0].estimatedEndTime, "download.estimatedEndTime is correct");
 });
 
-add_task(async function cleanup() {
-  await extension.unload();
+add_task(async function test_byExtension() {
+  let msg = await runInExtension("download", { url: TXT_URL });
+  equal(msg.status, "success", "download() succeeded");
+  const id = msg.result;
+  msg = await runInExtension("search", { id });
+
+  equal(msg.result.length, 1, "search() found 1 download");
+  equal(
+    msg.result[0].byExtensionName,
+    "Generated extension",
+    "download.byExtensionName is correct"
+  );
+  equal(
+    msg.result[0].byExtensionId,
+    extension.id,
+    "download.byExtensionId is correct"
+  );
 });
