@@ -8,11 +8,9 @@
 
 #include "nsFind.h"
 #include "mozilla/Likely.h"
-#include "nsContentCID.h"
 #include "nsIContent.h"
 #include "nsINode.h"
 #include "nsIFrame.h"
-#include "nsITextControlFrame.h"
 #include "nsIFormControl.h"
 #include "nsTextFragment.h"
 #include "nsString.h"
@@ -65,9 +63,9 @@ NS_IMPL_CYCLE_COLLECTION(nsFind)
 #  define DEBUG_FIND_PRINTF(...) /* nothing */
 #endif
 
-static nsIContent& AnonymousSubtreeRootParent(const nsINode& aNode) {
+static nsIContent& AnonymousSubtreeRootParentOrHost(const nsINode& aNode) {
   MOZ_ASSERT(aNode.IsInNativeAnonymousSubtree());
-  return *aNode.GetClosestNativeAnonymousSubtreeRootParent();
+  return *aNode.GetClosestNativeAnonymousSubtreeRootParentOrHost();
 }
 
 static void DumpNode(const nsINode* aNode) {
@@ -101,7 +99,15 @@ static bool IsBlockNode(const nsIContent* aContent) {
   }
 
   nsIFrame* frame = aContent->GetPrimaryFrame();
-  return frame && frame->StyleDisplay()->IsBlockOutsideStyle();
+  if (!frame) {
+    return false;
+  }
+
+  const auto& disp = *frame->StyleDisplay();
+  // We also treat internal table frames as "blocks" for the purpose of
+  // locating boundaries for searches (see
+  // https://bugzilla.mozilla.org/show_bug.cgi?id=1645990).
+  return disp.IsBlockOutsideStyle() || disp.IsInternalTableStyleExceptCell();
 }
 
 static bool IsDisplayedNode(const nsINode* aNode) {
@@ -132,7 +138,7 @@ static bool IsRubyAnnotationNode(const nsINode* aNode) {
          StyleDisplay::RubyTextContainer == display;
 }
 
-static bool IsVisibleNode(const nsINode* aNode) {
+static bool IsFindableNode(const nsINode* aNode) {
   if (!IsDisplayedNode(aNode)) {
     return false;
   }
@@ -142,19 +148,29 @@ static bool IsVisibleNode(const nsINode* aNode) {
     // display: contents
     return true;
   }
-
-  if (frame->IsContentHidden() || frame->AncestorHidesContent()) {
+  if (frame->StyleUI()->IsInert()) {
+    return false;
+  }
+  if (!frame->StyleVisibility()->IsVisible()) {
     return false;
   }
 
-  return frame->StyleVisibility()->IsVisible();
+  const bool isContentVisibilityHidden =
+      frame->HidesContent(nsIFrame::IncludeContentVisibility::Hidden) ||
+      frame->IsHiddenByContentVisibilityOnAnyAncestor(
+          nsIFrame::IncludeContentVisibility::Hidden);
+  if (isContentVisibilityHidden) {
+    return frame->IsHiddenUntilFoundOrClosedDetails();
+  }
+
+  return true;
 }
 
 static bool ShouldFindAnonymousContent(const nsIContent& aContent) {
   MOZ_ASSERT(aContent.IsInNativeAnonymousSubtree());
 
-  nsIContent& parent = AnonymousSubtreeRootParent(aContent);
-  if (nsCOMPtr<nsIFormControl> formControl = do_QueryInterface(&parent)) {
+  nsIContent& host = AnonymousSubtreeRootParentOrHost(aContent);
+  if (const auto* formControl = nsIFormControl::FromNode(&host)) {
     if (formControl->IsTextControl(/* aExcludePassword = */ true)) {
       // Only editable NAC in textfields should be findable. That is, we want to
       // find "bar" in `<input value="bar">`, but not in `<input
@@ -245,7 +261,7 @@ static bool ForceBreakBetweenText(const Text& aPrevious, const Text& aNext) {
 }
 
 struct nsFind::State final {
-  State(bool aFindBackward, nsIContent& aRoot, const nsRange& aStartPoint)
+  State(bool aFindBackward, nsIContent& aRoot, const RangeBoundary& aStartPoint)
       : mFindBackward(aFindBackward),
         mInitialized(false),
         mFoundBreak(false),
@@ -338,7 +354,7 @@ struct nsFind::State final {
   TreeIterator<StyleChildrenIterator> mIterator;
 
   // These are only needed for the first GetNextNode() call.
-  const nsRange& mStartPoint;
+  const RangeBoundary& mStartPoint;
 };
 
 void nsFind::State::Advance(Initializing aInitializing, bool aAlreadyMatching) {
@@ -368,12 +384,10 @@ void nsFind::State::Initialize() {
   mInitialized = true;
   mIterOffset = mFindBackward ? -1 : 0;
 
-  nsINode* container = mFindBackward ? mStartPoint.GetStartContainer()
-                                     : mStartPoint.GetEndContainer();
+  nsINode* container = mStartPoint.GetContainer();
 
   // Set up ourselves at the first node we want to start searching at.
-  nsIContent* beginning = mFindBackward ? mStartPoint.GetChildAtStartOffset()
-                                        : mStartPoint.GetChildAtEndOffset();
+  nsIContent* beginning = mStartPoint.GetChildAtOffset();
   if (beginning) {
     mIterator.Seek(*beginning);
     // If the start point is pointing to a node, when looking backwards we'd
@@ -405,8 +419,8 @@ void nsFind::State::Initialize() {
     return;
   }
 
-  mIterOffset =
-      mFindBackward ? mStartPoint.StartOffset() : mStartPoint.EndOffset();
+  mIterOffset = int(
+      *mStartPoint.Offset(RangeBoundary::OffsetFilter::kValidOrInvalidOffsets));
 }
 
 class MOZ_STACK_CLASS nsFind::StateRestorer final {
@@ -470,13 +484,13 @@ NS_IMETHODIMP
 nsFind::GetEntireWord(bool* aEntireWord) {
   if (!aEntireWord) return NS_ERROR_NULL_POINTER;
 
-  *aEntireWord = mEntireWord;
+  *aEntireWord = mWordStartBounded && mWordEndBounded;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFind::SetEntireWord(bool aEntireWord) {
-  mEntireWord = aEntireWord;
+  mWordStartBounded = mWordEndBounded = aEntireWord;
   return NS_OK;
 }
 
@@ -524,14 +538,14 @@ char32_t nsFind::DecodeChar(const char16_t* t2b, int32_t* index) const {
   return c;
 }
 
-bool nsFind::BreakInBetween(char32_t x, char32_t y) const {
+bool nsFind::BreakInBetween(char32_t x, char32_t y) {
   nsAutoStringN<4> text;
   AppendUCS4ToUTF16(x, text);
   const uint32_t x16Len = text.Length();
   AppendUCS4ToUTF16(y, text);
+  mWordBreakIter.Reset(text);
 
-  intl::WordBreakIteratorUtf16 iter(text);
-  return *iter.Seek(x16Len - 1) == x16Len;
+  return *mWordBreakIter.Seek(x16Len - 1) == x16Len;
 }
 
 char32_t nsFind::PeekNextChar(State& aState, bool aAlreadyMatching) const {
@@ -565,7 +579,16 @@ char32_t nsFind::PeekNextChar(State& aState, bool aAlreadyMatching) const {
 }
 
 #define NBSP_CHARCODE (CHAR_TO_UNICHAR(160))
-#define IsSpace(c) (nsCRT::IsAsciiSpace(c) || (c) == NBSP_CHARCODE)
+// Bug 1943846: The char16_t version of nsCRT::IsAsciiSpace is not working as
+// expected for characters with UTF32 > 0xFFFF. Add a check for UTF32 characters
+
+static bool IsSpace(char16_t aChar) {
+  return nsCRT::IsAsciiSpace(aChar) || aChar == NBSP_CHARCODE;
+}
+static bool IsSpace(char32_t aChar) {
+  return aChar <= 0xffff && IsSpace(char16_t(aChar));
+}
+
 #define OVERFLOW_PINDEX (mFindBackward ? pindex < 0 : pindex > patLen)
 #define DONE_WITH_PINDEX (mFindBackward ? pindex <= 0 : pindex >= patLen)
 
@@ -584,14 +607,49 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   NS_ENSURE_ARG(aEndPoint);
   NS_ENSURE_ARG_POINTER(aRangeRet);
 
-  Document* document =
-      aStartPoint->GetRoot() ? aStartPoint->GetRoot()->OwnerDoc() : nullptr;
-  NS_ENSURE_ARG(document);
+  *aRangeRet = nullptr;
+
+  if (RefPtr findResult = FindFromRangeBoundaries(
+          aPatText, aStartPoint->StartRef(), aEndPoint->EndRef())) {
+    findResult.forget(aRangeRet);
+  }
+  return NS_OK;
+}
+already_AddRefed<nsRange> nsFind::FindFromRangeBoundaries(
+    const nsAString& aPatText, const mozilla::RangeBoundary& aStartPoint,
+    const mozilla::RangeBoundary& aEndPoint) {
+  if (!aStartPoint.IsSetAndInComposedDoc() ||
+      !aEndPoint.IsSetAndInComposedDoc()) {
+    return nullptr;
+  }
+  MOZ_ASSERT(aStartPoint.IsSetAndValid());
+  MOZ_ASSERT(aEndPoint.IsSetAndValid());
+  // set up node index cache. If there is no cache given from the outside
+  // (for repeated calls to `Find()`), set up a local one.
+  nsContentUtils::NodeIndexCache localCache;
+  AutoRestore<nsContentUtils::NodeIndexCache*> restoreCache(mNodeIndexCache);
+  if (!mNodeIndexCache) {
+    mNodeIndexCache = &localCache;
+  }
+#if MOZ_DIAGNOSTIC_ASSERT_ENABLED
+  auto cmp =
+      nsContentUtils::ComparePoints(aStartPoint, aEndPoint, mNodeIndexCache);
+  MOZ_DIAGNOSTIC_ASSERT(cmp, "Start and end points in different trees?");
+  MOZ_DIAGNOSTIC_ASSERT(*cmp != 1, "Start point must not be after end point");
+#endif
+  if (MOZ_UNLIKELY(aStartPoint == aEndPoint)) {
+    return nullptr;
+  }
+
+  const Document* document = aStartPoint.GetContainer()->GetComposedDoc();
+  if (!document) {
+    return nullptr;
+  }
 
   Element* root = document->GetRootElement();
-  NS_ENSURE_ARG(root);
-
-  *aRangeRet = 0;
+  if (!root) {
+    return nullptr;
+  }
 
   nsAutoString patAutoStr(aPatText);
   if (!mCaseSensitive) {
@@ -606,11 +664,11 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   patAutoStr.StripChars(kShy);
 
   const char16_t* patStr = patAutoStr.get();
-  int32_t patLen = patAutoStr.Length() - 1;
+  const int32_t patLen = int(patAutoStr.Length()) - 1;
 
   // If this function is called with an empty string, we should early exit.
   if (patLen < 0) {
-    return NS_OK;
+    return nullptr;
   }
 
   const int32_t patternStart = mFindBackward ? patLen : 0;
@@ -622,7 +680,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   int32_t findex = 0;
 
   // Direction to move pindex and ptr*
-  int incr = mFindBackward ? -1 : 1;
+  const int incr = mFindBackward ? -1 : 1;
 
   const nsTextFragment* frag = nullptr;
   int32_t fragLen = 0;
@@ -641,14 +699,13 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
   char32_t matchAnchorChar = 0;
 
   // Get the end point, so we know when to end searches:
-  nsINode* endNode = aEndPoint->GetEndContainer();
-  uint32_t endOffset = aEndPoint->EndOffset();
+  const RangeBoundary& endPoint = mFindBackward ? aStartPoint : aEndPoint;
 
   char32_t c = 0;
   char32_t patc = 0;
   char32_t prevCharInMatch = 0;
 
-  State state(mFindBackward, *root, *aStartPoint);
+  State state(mFindBackward, *root, mFindBackward ? aEndPoint : aStartPoint);
   Text* current = nullptr;
 
   auto EndPartialMatch = [&]() -> bool {
@@ -697,7 +754,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
         if (EndPartialMatch()) {
           continue;
         }
-        return NS_OK;
+        return nullptr;
       }
 
       // We have a new text content. See if we need to force a break due to
@@ -713,7 +770,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
       }
 
       frag = &current->TextFragment();
-      fragLen = frag->GetLength();
+      fragLen = int32_t(frag->GetLength());
 
       // Set our starting point in this node. If we're going back to the anchor
       // node, which means that we just ended a partial match, use the saved
@@ -775,11 +832,13 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
 
     // Have we gone past the endpoint yet? If we have, and we're not in the
     // middle of a match, return.
-    if (state.GetCurrentNode() == endNode &&
-        ((mFindBackward && findex < static_cast<int32_t>(endOffset)) ||
-         (!mFindBackward && findex > static_cast<int32_t>(endOffset)))) {
-      DEBUG_FIND_PRINTF("Reached the end and not in the middle of a match\n");
-      return NS_OK;
+    if (auto cmp = nsContentUtils::ComparePoints(
+            RawRangeBoundary(state.GetCurrentNode(), findex), endPoint,
+            mNodeIndexCache)) {
+      if ((mFindBackward && *cmp < 0) || (!mFindBackward && *cmp > 0)) {
+        DEBUG_FIND_PRINTF("Reached the end and not in the middle of a match\n");
+        return nullptr;
+      }
     }
 
     // Save the previous character for word boundary detection
@@ -848,9 +907,10 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
       }
     }
 
-    // Figure whether the previous char is a word-breaking one.
-    bool wordBreakPrev = false;
-    if (mEntireWord) {
+    // Figure whether the previous char is a word-breaking one,
+    // if we care about word boundaries.
+    bool wordBreakPrev = true;
+    if (mWordStartBounded && prevChar) {
       if (prevChar == NBSP_CHARCODE) {
         prevChar = CHAR_TO_UNICHAR(' ');
       }
@@ -863,7 +923,8 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
     // b) a match has already been stored
     // c) the previous character is a different "class" than the current
     // character.
-    if ((c == patc && (!mEntireWord || matchAnchorNode || wordBreakPrev)) ||
+    if ((c == patc && (!(mWordStartBounded || mWordEndBounded) ||
+                       matchAnchorNode || wordBreakPrev)) ||
         (inWhitespace && IsSpace(c))) {
       prevCharInMatch = c;
       if (inWhitespace) {
@@ -890,7 +951,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
 
         // Make the range:
         // Check for word break (if necessary)
-        if (mEntireWord || inWhitespace) {
+        if (mWordEndBounded || inWhitespace) {
           int32_t nextfindex = findex + incr;
 
           char32_t nextChar;
@@ -911,7 +972,7 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
           }
 
           // If a word break isn't there when it needs to be, reset search.
-          if (mEntireWord && !BreakInBetween(c, nextChar)) {
+          if (mWordEndBounded && nextChar && !BreakInBetween(c, nextChar)) {
             matchAnchorNode = nullptr;
             continue;
           }
@@ -942,16 +1003,24 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
         }
 
         RefPtr<nsRange> range = nsRange::Create(current);
-        if (startParent && endParent && IsVisibleNode(startParent) &&
-            IsVisibleNode(endParent)) {
+        if (startParent && endParent && IsFindableNode(startParent) &&
+            IsFindableNode(endParent)) {
           IgnoredErrorResult rv;
           range->SetStart(*startParent, matchStartOffset, rv);
           if (!rv.Failed()) {
             range->SetEnd(*endParent, matchEndOffset, rv);
           }
+          // https://html.spec.whatwg.org/#interaction-with-details-and-hidden=until-found
+          NS_DispatchToMainThread(NS_NewRunnableFunction(
+              "RevealHiddenUntilFound",
+              [node = RefPtr(startParent)]()
+                  MOZ_CAN_RUN_SCRIPT_BOUNDARY_LAMBDA {
+                    node->RevealAncestorClosedDetails();
+                    node->RevealAncestorHiddenUntilFoundAndFireBeforematchEvent(
+                        IgnoreErrors());
+                  }));
           if (!rv.Failed()) {
-            range.forget(aRangeRet);
-            return NS_OK;
+            return range.forget();
           }
         }
 
@@ -976,6 +1045,4 @@ nsFind::Find(const nsAString& aPatText, nsRange* aSearchRange,
     DEBUG_FIND_PRINTF("NOT: %c == %c\n", c, patc);
     EndPartialMatch();
   }
-
-  return NS_OK;
 }
