@@ -15,25 +15,14 @@ add_task(async function test_clickInsecureFieldWarning() {
       gBrowser,
       url,
     },
-    async function(browser) {
+    async function (browser) {
       let popup = document.getElementById("PopupAutoComplete");
-      ok(popup, "Got popup");
-
-      let promiseShown = BrowserTestUtils.waitForEvent(popup, "popupshown");
-
-      await SimpleTest.promiseFocus(browser);
-      info("content window focused");
-
-      // Focus the username field to open the popup.
-      await ContentTask.spawn(browser, null, function openAutocomplete() {
-        content.document.getElementById("form-basic-username").focus();
-      });
-
-      await promiseShown;
-      ok(promiseShown, "autocomplete shown");
+      Assert.ok(popup, "Got popup");
+      await openACPopup(popup, browser, "#form-basic-username");
+      await new Promise(requestAnimationFrame);
 
       let warningItem = popup.querySelector(`[type="insecureWarning"]`);
-      ok(warningItem, "Got warning richlistitem");
+      Assert.ok(warningItem, "Got warning richlistitem");
 
       await BrowserTestUtils.waitForCondition(
         () => !warningItem.collapsed,
@@ -47,7 +36,8 @@ add_task(async function test_clickInsecureFieldWarning() {
       );
       EventUtils.synthesizeMouseAtCenter(warningItem, {});
       let supportTab = await supportTabPromise;
-      ok(supportTab, "Support tab opened");
+      Assert.ok(supportTab, "Support tab opened");
+      await closePopup(popup);
       BrowserTestUtils.removeTab(supportTab);
     }
   );
