@@ -16,8 +16,6 @@ use xpcom::interfaces::{mozISyncedBookmarksMerger, nsINavBookmarksService};
 use crate::driver::{AbortController, Driver};
 use crate::error::{Error, Result};
 
-pub const LMANNO_FEEDURI: &'static str = "livemark/feedURI";
-
 extern "C" {
     fn NS_NavBookmarksTotalSyncChanges() -> i64;
 }
@@ -85,7 +83,7 @@ impl<'s> Store<'s> {
             "SELECT NOT EXISTS(
                SELECT 1 FROM moz_bookmarks
                WHERE id = (SELECT parent FROM moz_bookmarks
-                           WHERE guid = '{0}')
+                           WHERE guid = '{root}')
              ) AND NOT EXISTS(
                SELECT 1 FROM moz_bookmarks b
                JOIN moz_bookmarks p ON p.id = b.parent
@@ -215,7 +213,7 @@ impl<'s> Store<'s> {
                         let title = String::from_utf16(&*raw_title)?;
                         url.map(|url| Content::Bookmark {
                             title,
-                            url_href: url.into_string(),
+                            url_href: url.into(),
                         })
                     }
                     Kind::Folder | Kind::Livemark => {
@@ -321,15 +319,11 @@ impl<'s> dogear::Store for Store<'s> {
                     b.syncStatus, b.lastModified / 1000 AS localModified,
                     IFNULL(b.title, '') AS title,
                     (SELECT h.url FROM moz_places h WHERE h.id = b.fk) AS url,
-                    EXISTS(SELECT 1 FROM moz_items_annos a
-                           JOIN moz_anno_attributes n ON n.id = a.anno_attribute_id
-                           WHERE a.item_id = b.id AND
-                                 n.name = '{}') AS isLivemark
+                    0 AS isLivemark
              FROM moz_bookmarks b
              JOIN moz_bookmarks p ON p.id = b.parent
              WHERE b.guid <> '{}'
              ORDER BY b.parent, b.position",
-            LMANNO_FEEDURI,
             dogear::ROOT_GUID,
         ))?;
         while let Some(step) = items_statement.step()? {
@@ -537,7 +531,8 @@ fn update_local_items_in_places<'t>(
         let mut statement = db.prepare(format!(
             "INSERT OR IGNORE INTO moz_places(url, url_hash, rev_host, hidden,
                                               frecency, guid)
-             SELECT u.url, u.hash, u.revHost, 0,
+             SELECT u.url, u.hash, u.revHost,
+                    (CASE WHEN u.url BETWEEN 'place:' AND 'place:' || X'FFFF' THEN 1 ELSE 0 END),
                     (CASE v.kind WHEN {} THEN 0 ELSE -1 END),
                     IFNULL((SELECT h.guid FROM moz_places h
                             WHERE h.url_hash = u.hash AND
@@ -556,11 +551,6 @@ fn update_local_items_in_places<'t>(
         statement.execute()?;
     }
 
-    // Trigger frecency updates for all new origins.
-    debug!(driver, "Updating origins for new URLs");
-    controller.err_if_aborted()?;
-    db.exec("DELETE FROM moz_updateoriginsinsert_temp")?;
-
     // Build a table of new and updated items.
     debug!(driver, "Staging apply remote item ops");
     for chunk in ops.apply_remote_items.chunks(db.variable_limit()? / 3) {
@@ -752,11 +742,6 @@ fn update_local_items_in_places<'t>(
     debug!(driver, "Applying remote items");
     apply_remote_items(db, driver, controller)?;
 
-    // Trigger frecency updates for all affected origins.
-    debug!(driver, "Updating origins for changed URLs");
-    controller.err_if_aborted()?;
-    db.exec("DELETE FROM moz_updateoriginsupdate_temp")?;
-
     // Fires the `applyNewLocalStructure` trigger.
     debug!(driver, "Applying new local structure");
     controller.err_if_aborted()?;
@@ -934,8 +919,8 @@ fn apply_remote_items(db: &Conn, driver: &Driver, controller: &AbortController)
     controller.err_if_aborted()?;
     db.exec(
         "UPDATE moz_places SET
-           frecency = -frecency
-         WHERE frecency > 0 AND (
+           recalc_frecency = 1, recalc_alt_frecency = 1
+         WHERE frecency <> 0 AND (
            id IN (
              SELECT oldPlaceId FROM itemsToApply
              WHERE oldPlaceId <> newPlaceId
@@ -979,9 +964,9 @@ fn remove_local_items(
         "WITH
          ops(guid, level) AS (VALUES {})
          INSERT INTO itemsRemoved(itemId, parentId, position, type, title,
-                                  placeId, guid, parentGuid, level)
+                                  placeId, guid, parentGuid, level, keywordRemoved)
          SELECT b.id, b.parent, b.position, b.type, IFNULL(b.title, \"\"), b.fk,
-                b.guid, p.guid, n.level
+                b.guid, p.guid, n.level, EXISTS(SELECT 1 FROM moz_keywords k WHERE k.place_id = b.fk)
          FROM ops n
          JOIN moz_bookmarks b ON b.guid = n.guid
          JOIN moz_bookmarks p ON p.id = b.parent",
@@ -1002,10 +987,10 @@ fn remove_local_items(
     debug!(driver, "Recalculating frecencies for removed bookmark URLs");
     let mut frecency_statement = db.prepare(format!(
         "UPDATE moz_places SET
-           frecency = -frecency
+            recalc_frecency = 1, recalc_alt_frecency = 1
          WHERE id IN (SELECT b.fk FROM moz_bookmarks b
                       WHERE b.guid IN ({})) AND
-               frecency > 0",
+               frecency <> 0",
         repeat_sql_vars(ops.len())
     ))?;
     for (index, op) in ops.iter().enumerate() {
@@ -1034,6 +1019,25 @@ fn remove_local_items(
     }
     annos_statement.execute()?;
 
+    debug!(
+        driver,
+        "Removing keywords associated with deleted bookmarks"
+    );
+    let mut keywords_statement = db.prepare(format!(
+        "DELETE FROM moz_keywords
+         WHERE place_id IN (SELECT b.fk FROM moz_bookmarks b
+            WHERE b.guid IN ({}))",
+        repeat_sql_vars(ops.len()),
+    ))?;
+    for (index, op) in ops.iter().enumerate() {
+        controller.err_if_aborted()?;
+        keywords_statement.bind_by_index(
+            index as u32,
+            nsString::from(&*op.local_node().guid.as_str()),
+        )?;
+    }
+    keywords_statement.execute()?;
+
     debug!(driver, "Removing deleted items from Places");
     let mut delete_statement = db.prepare(format!(
         "DELETE FROM moz_bookmarks
@@ -1090,7 +1094,8 @@ fn stage_items_to_upload(
         "INSERT OR IGNORE INTO itemsToUpload(id, guid, syncChangeCounter,
                                              parentGuid, parentTitle, dateAdded,
                                              type, title, placeId, isQuery, url,
-                                             keyword, position, tagFolderName)
+                                             keyword, position, tagFolderName,
+                                             unknownFields)
          {}
          JOIN itemsToApply n ON n.mergedGuid = b.guid
          WHERE n.localDateAddedMicroseconds < n.remoteDateAddedMicroseconds",
@@ -1104,7 +1109,8 @@ fn stage_items_to_upload(
                                                  parentGuid, parentTitle,
                                                  dateAdded, type, title,
                                                  placeId, isQuery, url, keyword,
-                                                 position, tagFolderName)
+                                                 position, tagFolderName,
+                                                 unknownFields)
              {}
              WHERE b.guid IN ({})",
             UploadItemsFragment("b"),
@@ -1213,12 +1219,11 @@ trait Column<T> {
     fn from_column(raw: T) -> Result<Self>
     where
         Self: Sized;
-    fn into_column(self) -> T;
 }
 
 impl Column<i64> for Kind {
     fn from_column(raw: i64) -> Result<Kind> {
-        Ok(match i16::try_from(raw) {
+        Ok(match u8::try_from(raw) {
             Ok(mozISyncedBookmarksMerger::KIND_BOOKMARK) => Kind::Bookmark,
             Ok(mozISyncedBookmarksMerger::KIND_QUERY) => Kind::Query,
             Ok(mozISyncedBookmarksMerger::KIND_FOLDER) => Kind::Folder,
@@ -1227,35 +1232,17 @@ impl Column<i64> for Kind {
             _ => return Err(Error::UnknownItemKind(raw)),
         })
     }
-
-    fn into_column(self) -> i64 {
-        match self {
-            Kind::Bookmark => mozISyncedBookmarksMerger::KIND_BOOKMARK as i64,
-            Kind::Query => mozISyncedBookmarksMerger::KIND_QUERY as i64,
-            Kind::Folder => mozISyncedBookmarksMerger::KIND_FOLDER as i64,
-            Kind::Livemark => mozISyncedBookmarksMerger::KIND_LIVEMARK as i64,
-            Kind::Separator => mozISyncedBookmarksMerger::KIND_SEPARATOR as i64,
-        }
-    }
 }
 
 impl Column<i64> for Validity {
     fn from_column(raw: i64) -> Result<Validity> {
-        Ok(match i16::try_from(raw) {
+        Ok(match u8::try_from(raw) {
             Ok(mozISyncedBookmarksMerger::VALIDITY_VALID) => Validity::Valid,
             Ok(mozISyncedBookmarksMerger::VALIDITY_REUPLOAD) => Validity::Reupload,
             Ok(mozISyncedBookmarksMerger::VALIDITY_REPLACE) => Validity::Replace,
             _ => return Err(Error::UnknownItemValidity(raw).into()),
         })
     }
-
-    fn into_column(self) -> i64 {
-        match self {
-            Validity::Valid => mozISyncedBookmarksMerger::VALIDITY_VALID as i64,
-            Validity::Reupload => mozISyncedBookmarksMerger::VALIDITY_REUPLOAD as i64,
-            Validity::Replace => mozISyncedBookmarksMerger::VALIDITY_REPLACE as i64,
-        }
-    }
 }
 
 /// Formats an optional value so that it can be included in a SQL statement.
@@ -1290,10 +1277,12 @@ impl fmt::Display for UploadItemsFragment {
                        (SELECT keyword FROM moz_keywords WHERE place_id = h.id),
                        {0}.position,
                        (SELECT get_query_param(substr(url, 7), 'tag')
-                        WHERE substr(h.url, 1, 6) = 'place:') AS tagFolderName
+                        WHERE substr(h.url, 1, 6) = 'place:') AS tagFolderName,
+                        v.unknownFields
                 FROM moz_bookmarks {0}
                 JOIN moz_bookmarks p ON p.id = {0}.parent
-                LEFT JOIN moz_places h ON h.id = {0}.fk",
+                LEFT JOIN moz_places h ON h.id = {0}.fk
+                LEFT JOIN items v ON v.guid = {0}.guid",
             self.0
         )
     }
