@@ -7,109 +7,65 @@
 var bs = PlacesUtils.bookmarks;
 var hs = PlacesUtils.history;
 var os = PlacesUtils.observers;
-var anno = PlacesUtils.annotations;
 
 var bookmarksObserver = {
   handlePlacesEvents(events) {
     Assert.equal(events.length, 1);
     let event = events[0];
-    bookmarksObserver._itemAddedId = event.id;
-    bookmarksObserver._itemAddedParent = event.parentId;
-    bookmarksObserver._itemAddedIndex = event.index;
-    bookmarksObserver._itemAddedURI = event.url
-      ? Services.io.newURI(event.url)
-      : null;
-    bookmarksObserver._itemAddedTitle = event.title;
+    switch (event.type) {
+      case "bookmark-added": {
+        bookmarksObserver._itemAddedId = event.id;
+        bookmarksObserver._itemAddedParent = event.parentId;
+        bookmarksObserver._itemAddedIndex = event.index;
+        bookmarksObserver._itemAddedURI = event.url
+          ? Services.io.newURI(event.url)
+          : null;
+        bookmarksObserver._itemAddedTitle = event.title;
 
-    // Ensure that we've created a guid for this item.
-    let stmt = DBConn().createStatement(
-      `SELECT guid
-       FROM moz_bookmarks
-       WHERE id = :item_id`
-    );
-    stmt.params.item_id = event.id;
-    Assert.ok(stmt.executeStep());
-    Assert.ok(!stmt.getIsNull(0));
-    do_check_valid_places_guid(stmt.row.guid);
-    Assert.equal(stmt.row.guid, event.guid);
-    stmt.finalize();
+        // Ensure that we've created a guid for this item.
+        let stmt = DBConn().createStatement(
+          `SELECT guid
+           FROM moz_bookmarks
+           WHERE id = :item_id`
+        );
+        stmt.params.item_id = event.id;
+        Assert.ok(stmt.executeStep());
+        Assert.ok(!stmt.getIsNull(0));
+        do_check_valid_places_guid(stmt.row.guid);
+        Assert.equal(stmt.row.guid, event.guid);
+        stmt.finalize();
+        break;
+      }
+      case "bookmark-removed":
+        bookmarksObserver._itemRemovedId = event.id;
+        bookmarksObserver._itemRemovedFolder = event.parentId;
+        bookmarksObserver._itemRemovedIndex = event.index;
+        break;
+      case "bookmark-title-changed":
+        bookmarksObserver._itemTitleChangedId = event.id;
+        bookmarksObserver._itemTitleChangedTitle = event.title;
+        break;
+    }
   },
-
-  onBeginUpdateBatch() {
-    this._beginUpdateBatch = true;
-  },
-  onEndUpdateBatch() {
-    this._endUpdateBatch = true;
-  },
-  onItemRemoved(id, folder, index, itemType) {
-    this._itemRemovedId = id;
-    this._itemRemovedFolder = folder;
-    this._itemRemovedIndex = index;
-  },
-  onItemChanged(
-    id,
-    property,
-    isAnnotationProperty,
-    value,
-    lastModified,
-    itemType,
-    parentId,
-    guid,
-    parentGuid,
-    oldValue
-  ) {
-    this._itemChangedId = id;
-    this._itemChangedProperty = property;
-    this._itemChanged_isAnnotationProperty = isAnnotationProperty;
-    this._itemChangedValue = value;
-    this._itemChangedOldValue = oldValue;
-  },
-  onItemVisited(id, visitID, time) {
-    this._itemVisitedId = id;
-    this._itemVisitedVistId = visitID;
-    this._itemVisitedTime = time;
-  },
-  onItemMoved(id, oldParent, oldIndex, newParent, newIndex, itemType) {
-    this._itemMovedId = id;
-    this._itemMovedOldParent = oldParent;
-    this._itemMovedOldIndex = oldIndex;
-    this._itemMovedNewParent = newParent;
-    this._itemMovedNewIndex = newIndex;
-  },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsINavBookmarkObserver]),
 };
 
-// Get bookmarks menu folder id.
-var root = bs.bookmarksMenuFolder;
+var root;
 // Index at which items should begin.
 var bmStartIndex = 0;
 
+add_task(async function setup() {
+  // Get bookmarks menu folder id.
+  root = await PlacesTestUtils.promiseItemId(PlacesUtils.bookmarks.menuGuid);
+});
+
 add_task(async function test_bookmarks() {
-  bs.addObserver(bookmarksObserver);
-  os.addListener(["bookmark-added"], bookmarksObserver.handlePlacesEvents);
+  os.addListener(
+    ["bookmark-added", "bookmark-removed", "bookmark-title-changed"],
+    bookmarksObserver.handlePlacesEvents
+  );
 
   // test special folders
-  Assert.ok(bs.placesRoot > 0);
-  Assert.ok(bs.bookmarksMenuFolder > 0);
   Assert.ok(bs.tagsFolder > 0);
-  Assert.ok(bs.toolbarFolder > 0);
-
-  // test getFolderIdForItem() with bogus item id will throw
-  try {
-    bs.getFolderIdForItem(0);
-    do_throw("getFolderIdForItem accepted bad input");
-  } catch (ex) {}
-
-  // test getFolderIdForItem() with bogus item id will throw
-  try {
-    bs.getFolderIdForItem(-1);
-    do_throw("getFolderIdForItem accepted bad input");
-  } catch (ex) {}
-
-  // test root parentage
-  Assert.equal(bs.getFolderIdForItem(bs.bookmarksMenuFolder), bs.placesRoot);
-  Assert.equal(bs.getFolderIdForItem(bs.tagsFolder), bs.placesRoot);
-  Assert.equal(bs.getFolderIdForItem(bs.toolbarFolder), bs.placesRoot);
 
   // create a folder to hold all the tests
   // this makes the tests more tolerant of changes to default_places.html
@@ -118,7 +74,7 @@ add_task(async function test_bookmarks() {
     "places bookmarks xpcshell tests",
     bs.DEFAULT_INDEX
   );
-  let testRootGuid = await PlacesUtils.promiseItemGuid(testRoot);
+  let testRootGuid = await PlacesTestUtils.promiseItemGuid(testRoot);
   Assert.equal(bookmarksObserver._itemAddedId, testRoot);
   Assert.equal(bookmarksObserver._itemAddedParent, root);
   Assert.equal(bookmarksObserver._itemAddedIndex, bmStartIndex);
@@ -143,9 +99,11 @@ add_task(async function test_bookmarks() {
 
   // after just inserting, modified should not be set
   let lastModified = PlacesUtils.toPRTime(
-    (await PlacesUtils.bookmarks.fetch(
-      await PlacesUtils.promiseItemGuid(newId)
-    )).lastModified
+    (
+      await PlacesUtils.bookmarks.fetch(
+        await PlacesTestUtils.promiseItemGuid(newId)
+      )
+    ).lastModified
   );
 
   // The time before we set the title, in microseconds.
@@ -159,15 +117,16 @@ add_task(async function test_bookmarks() {
 
   // set bookmark title
   bs.setItemTitle(newId, "Google");
-  Assert.equal(bookmarksObserver._itemChangedId, newId);
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
-  Assert.equal(bookmarksObserver._itemChangedValue, "Google");
+  Assert.equal(bookmarksObserver._itemTitleChangedId, newId);
+  Assert.equal(bookmarksObserver._itemTitleChangedTitle, "Google");
 
   // check lastModified after we set the title
   let lastModified2 = PlacesUtils.toPRTime(
-    (await PlacesUtils.bookmarks.fetch(
-      await PlacesUtils.promiseItemGuid(newId)
-    )).lastModified
+    (
+      await PlacesUtils.bookmarks.fetch(
+        await PlacesTestUtils.promiseItemGuid(newId)
+      )
+    ).lastModified
   );
   info("test setItemTitle");
   info("beforeSetTitle = " + beforeSetTitle);
@@ -185,10 +144,6 @@ add_task(async function test_bookmarks() {
     do_throw("getItemTitle accepted bad input");
   } catch (ex) {}
 
-  // get the folder that the bookmark is in
-  let folderId = bs.getFolderIdForItem(newId);
-  Assert.equal(folderId, testRoot);
-
   // create a folder at a specific index
   let workFolder = bs.createFolder(testRoot, "Work", 0);
   Assert.equal(bookmarksObserver._itemAddedId, workFolder);
@@ -213,7 +168,8 @@ add_task(async function test_bookmarks() {
 
   // change item
   bs.setItemTitle(newId2, "DevMo");
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
+  Assert.equal(bookmarksObserver._itemTitleChangedId, newId2);
+  Assert.equal(bookmarksObserver._itemTitleChangedTitle, "DevMo");
 
   // insert item into subfolder
   let newId3 = bs.insertBookmark(
@@ -228,7 +184,8 @@ add_task(async function test_bookmarks() {
 
   // change item
   bs.setItemTitle(newId3, "MSDN");
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
+  Assert.equal(bookmarksObserver._itemTitleChangedId, newId3);
+  Assert.equal(bookmarksObserver._itemTitleChangedTitle, "MSDN");
 
   // remove item
   bs.removeItem(newId2);
@@ -266,8 +223,8 @@ add_task(async function test_bookmarks() {
 
   // change item
   bs.setItemTitle(newId5, "ESPN");
-  Assert.equal(bookmarksObserver._itemChangedId, newId5);
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
+  Assert.equal(bookmarksObserver._itemTitleChangedId, newId5);
+  Assert.equal(bookmarksObserver._itemTitleChangedTitle, "ESPN");
 
   // insert query item
   let uri6 = uri(
@@ -280,7 +237,8 @@ add_task(async function test_bookmarks() {
 
   // change item
   bs.setItemTitle(newId6, "Google Sites");
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
+  Assert.equal(bookmarksObserver._itemTitleChangedId, newId6);
+  Assert.equal(bookmarksObserver._itemTitleChangedTitle, "Google Sites");
 
   // test bookmark id in query output
   try {
@@ -318,7 +276,7 @@ add_task(async function test_bookmarks() {
     let mURI = uri("http://multiple.uris.in.query");
 
     let testFolder = bs.createFolder(testRoot, "test Folder", bs.DEFAULT_INDEX);
-    let testFolderGuid = await PlacesUtils.promiseItemGuid(testFolder);
+    let testFolderGuid = await PlacesTestUtils.promiseItemGuid(testFolder);
     // add 2 bookmarks
     bs.insertBookmark(testFolder, mURI, bs.DEFAULT_INDEX, "title 1");
     bs.insertBookmark(testFolder, mURI, bs.DEFAULT_INDEX, "title 2");
@@ -368,23 +326,8 @@ add_task(async function test_bookmarks() {
 
   // set bookmark title
   bs.setItemTitle(newId13, "ZZZXXXYYY");
-  Assert.equal(bookmarksObserver._itemChangedId, newId13);
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
-  Assert.equal(bookmarksObserver._itemChangedValue, "ZZZXXXYYY");
-
-  // check if setting an item annotation triggers onItemChanged
-  bookmarksObserver._itemChangedId = -1;
-  anno.setItemAnnotation(
-    newId3,
-    "test-annotation",
-    "foo",
-    0,
-    anno.EXPIRE_NEVER
-  );
-  Assert.equal(bookmarksObserver._itemChangedId, newId3);
-  Assert.equal(bookmarksObserver._itemChangedProperty, "test-annotation");
-  Assert.ok(bookmarksObserver._itemChanged_isAnnotationProperty);
-  Assert.equal(bookmarksObserver._itemChangedValue, "");
+  Assert.equal(bookmarksObserver._itemTitleChangedId, newId13);
+  Assert.equal(bookmarksObserver._itemTitleChangedTitle, "ZZZXXXYYY");
 
   // test search on bookmark title ZZZXXXYYY
   try {
@@ -469,17 +412,14 @@ add_task(async function test_bookmarks() {
   );
   bs.setItemLastModified(newId14, 1234000000000000);
   let fakeLastModified = PlacesUtils.toPRTime(
-    (await PlacesUtils.bookmarks.fetch(
-      await PlacesUtils.promiseItemGuid(newId14)
-    )).lastModified
+    (
+      await PlacesUtils.bookmarks.fetch(
+        await PlacesTestUtils.promiseItemGuid(newId14)
+      )
+    ).lastModified
   );
   Assert.equal(fakeLastModified, 1234000000000000);
 
-  // ensure that removing an item removes its annotations
-  Assert.ok(anno.itemHasAnnotation(newId3, "test-annotation"));
-  bs.removeItem(newId3);
-  Assert.ok(!anno.itemHasAnnotation(newId3, "test-annotation"));
-
   // bug 378820
   let uri1 = uri("http://foo.tld/a");
   bs.insertBookmark(testRoot, uri1, bs.DEFAULT_INDEX, "");
@@ -500,9 +440,8 @@ add_task(async function test_bookmarks() {
   // test title length after updates
   bs.setItemTitle(newId15, title15 + " updated");
   Assert.equal(bs.getItemTitle(newId15).length, title15expected.length);
-  Assert.equal(bookmarksObserver._itemChangedId, newId15);
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
-  Assert.equal(bookmarksObserver._itemChangedValue, title15expected);
+  Assert.equal(bookmarksObserver._itemTitleChangedId, newId15);
+  Assert.equal(bookmarksObserver._itemTitleChangedTitle, title15expected);
 
   await testSimpleFolderResult();
 });
@@ -515,7 +454,7 @@ async function testSimpleFolderResult() {
 
   // create a folder
   let parent = bs.createFolder(root, "test", bs.DEFAULT_INDEX);
-  let parentGuid = await PlacesUtils.promiseItemGuid(parent);
+  let parentGuid = await PlacesTestUtils.promiseItemGuid(parent);
 
   // the time before we insert, in microseconds
   // Workaround possible VM timers issues subtracting 1ms.
@@ -568,10 +507,9 @@ async function testSimpleFolderResult() {
 
   // update with another long title
   bs.setItemTitle(folderLongName, longName + " updated");
-  Assert.equal(bookmarksObserver._itemChangedId, folderLongName);
-  Assert.equal(bookmarksObserver._itemChangedProperty, "title");
+  Assert.equal(bookmarksObserver._itemTitleChangedId, folderLongName);
   Assert.equal(
-    bookmarksObserver._itemChangedValue,
+    bookmarksObserver._itemTitleChangedTitle,
     longName.substring(0, TITLE_LENGTH_MAX)
   );
 
