@@ -5,13 +5,11 @@
 
 #include "reflect.h"
 #include "jsapi.h"
-#include "nsMemory.h"
+
 #include "nsString.h"
-#include "nsNativeCharsetUtils.h"
 #include "xpc_make_class.h"
 
-namespace mozilla {
-namespace reflect {
+namespace mozilla::reflect {
 
 NS_IMPL_ISUPPORTS(Module, nsIXPCScriptable)
 
@@ -35,5 +33,4 @@ Module::Call(nsIXPConnectWrappedNative* wrapper, JSContext* cx, JSObject* obj,
   return NS_OK;
 }
 
-}  // namespace reflect
-}  // namespace mozilla
+}  // namespace mozilla::reflect
