@@ -35,7 +35,7 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/Services.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/ReputationserviceMetrics.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/intl/LocaleService.h"
 
@@ -60,14 +60,11 @@
 
 #include "ApplicationReputationTelemetryUtils.h"
 
-using mozilla::ArrayLength;
 using mozilla::BasePrincipal;
 using mozilla::OriginAttributes;
 using mozilla::Preferences;
 using mozilla::TimeStamp;
 using mozilla::intl::LocaleService;
-using mozilla::Telemetry::Accumulate;
-using mozilla::Telemetry::AccumulateCategorical;
 using safe_browsing::ClientDownloadRequest;
 using safe_browsing::ClientDownloadRequest_CertificateChain;
 using safe_browsing::ClientDownloadRequest_Resource;
@@ -152,6 +149,7 @@ const char* const ApplicationReputationService::kNonBinaryExecutables[] = {
     ".air",
     ".atloc",
     ".ftploc",
+    ".terminal",
     // clang-format on
 };
 
@@ -181,6 +179,8 @@ const char* const ApplicationReputationService::kBinaryFileExtensions[] = {
     ".applescript",
     //".application", exec // MS ClickOnce
     //".appref-ms", exec // MS ClickOnce
+    //".appx", exec
+    //".appxbundle", exec
     //".arc",
     //".arj",
     ".as",  // Mac archive
@@ -276,9 +276,12 @@ const char* const ApplicationReputationService::kBinaryFileExtensions[] = {
     ".internetconnect",  // Configuration file for Apple system
     //".inx", // InstallShield
     ".iso",  // CD image
-    //".isp", exec // IIS config
-    //".isu", // InstallShield
-    //".jar", exec // Java
+             //".isp", exec // IIS config
+             //".isu", // InstallShield
+             //".jar", exec // Java
+#ifndef MOZ_ESR
+//".jnlp", exec // Java
+#endif
     //".job", // Windows
     //".jpg",
     //".jpeg",
@@ -286,6 +289,7 @@ const char* const ApplicationReputationService::kBinaryFileExtensions[] = {
     //".jse", exec // JScript
     ".ksh",  // Linux shell
     //".lha",
+    //".library-ms", exec // Windows
     //".lnk", exec // Windows
     ".local",  // Windows
     //".lpaq1",
@@ -331,6 +335,8 @@ const char* const ApplicationReputationService::kBinaryFileExtensions[] = {
     //".msh2xml", exec // Windows shell
     //".mshxml", exec // Windows
     //".msi", exec  // Windows installer
+    //".msix", exec // Windows installer
+    //".msixbundle", exec // Windows installer
     //".msp", exec  // Windows installer
     //".mst", exec  // Windows installer
     ".ndif",            // Mac disk image
@@ -512,20 +518,20 @@ const char* const ApplicationReputationService::kBinaryFileExtensions[] = {
     ".xlam",  // MS Excel
     ".xldm",  // MS Excel
     //".xll", exec  // MS Excel
-    ".xlm",     // MS Excel
-    ".xls",     // MS Excel
-    ".xlsb",    // MS Excel
-    ".xlsm",    // MS Excel
-    ".xlsx",    // MS Excel
-    ".xlt",     // MS Excel
-    ".xltm",    // MS Excel
-    ".xltx",    // MS Excel
-    ".xlw",     // MS Excel
-    ".xml",     // MS Excel
-    ".xnk",     // MS Exchange
-    ".xrm-ms",  // Windows
-    ".xsd",     // XML schema definition
-    ".xsl",     // XML Stylesheet
+    ".xlm",   // MS Excel
+    ".xls",   // MS Excel
+    ".xlsb",  // MS Excel
+    ".xlsm",  // MS Excel
+    ".xlsx",  // MS Excel
+    ".xlt",   // MS Excel
+    ".xltm",  // MS Excel
+    ".xltx",  // MS Excel
+    ".xlw",   // MS Excel
+    ".xml",   // MS Excel
+    ".xnk",   // MS Exchange
+    //".xrm-ms", exec // Windows
+    ".xsd",  // XML schema definition
+    ".xsl",  // XML Stylesheet
     //".xxe",
     ".xz",     // Linux archive (xz)
     ".z",      // InstallShield
@@ -584,7 +590,7 @@ enum class LookupType { AllowlistOnly, BlocklistOnly, BothLists };
 // Remote lookup is disabled or the remote lookup URL is empty InternalError :
 // An unexpected internal error DPDisabled           : Download protection is
 // disabled
-using Reason = mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_REASON;
+using Reason = mozilla::glean::application_reputation::ReasonLabel;
 
 class PendingDBLookup;
 
@@ -845,6 +851,9 @@ nsresult PendingDBLookup::LookupSpecInternal(const nsACString& aSpec) {
     }
     tables.Append(blocklist);
   }
+  if (principal->IsSystemPrincipal()) {
+    return mPendingLookup->LookupNext();
+  }
   return dbService->Lookup(principal, tables, this);
 }
 
@@ -859,11 +868,12 @@ PendingDBLookup::HandleEvent(const nsACString& tables) {
   if ((mLookupType != LookupType::AllowlistOnly) &&
       FindInReadable(blockList, tables)) {
     mPendingLookup->mBlocklistCount++;
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_LOCAL, BLOCK_LIST);
+    mozilla::glean::application_reputation::local.AccumulateSingleSample(
+        BLOCK_LIST);
     LOG(("Found principal %s on blocklist [this = %p]", mSpec.get(), this));
     return mPendingLookup->OnComplete(
         nsIApplicationReputationService::VERDICT_DANGEROUS,
-        Reason::LocalBlocklist, NS_OK);
+        Reason::eLocalblocklist, NS_OK);
   }
 
   nsAutoCString allowList;
@@ -871,14 +881,15 @@ PendingDBLookup::HandleEvent(const nsACString& tables) {
   if ((mLookupType != LookupType::BlocklistOnly) &&
       FindInReadable(allowList, tables)) {
     mPendingLookup->mAllowlistCount++;
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_LOCAL, ALLOW_LIST);
+    mozilla::glean::application_reputation::local.AccumulateSingleSample(
+        ALLOW_LIST);
     LOG(("Found principal %s on allowlist [this = %p]", mSpec.get(), this));
     // Don't call onComplete, since blocklisting trumps allowlisting
     return mPendingLookup->LookupNext();
   }
 
   LOG(("Didn't find principal %s on any list [this = %p]", mSpec.get(), this));
-  Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_LOCAL, NO_LIST);
+  mozilla::glean::application_reputation::local.AccumulateSingleSample(NO_LIST);
   return mPendingLookup->LookupNext();
 }
 
@@ -931,7 +942,7 @@ static const char* GetFileExt(const nsACString& aFilename,
 }
 
 static const char* GetFileExt(const nsACString& aFilename) {
-#define _GetFileExt(_f, _l) GetFileExt(_f, _l, ArrayLength(_l))
+#define _GetFileExt(_f, _l) GetFileExt(_f, _l, std::size(_l))
   const char* ext = _GetFileExt(
       aFilename, ApplicationReputationService::kBinaryFileExtensions);
   if (ext == nullptr &&
@@ -950,15 +961,13 @@ static bool IsFileType(const nsACString& aFilename,
 }
 
 static bool IsBinary(const nsACString& aFilename) {
-  return IsFileType(aFilename,
-                    ApplicationReputationService::kBinaryFileExtensions,
-                    ArrayLength(
-                        ApplicationReputationService::kBinaryFileExtensions)) ||
+  return IsFileType(
+             aFilename, ApplicationReputationService::kBinaryFileExtensions,
+             std::size(ApplicationReputationService::kBinaryFileExtensions)) ||
          (!IsFileType(
               aFilename, ApplicationReputationService::kNonBinaryExecutables,
-              ArrayLength(
-                  ApplicationReputationService::kNonBinaryExecutables)) &&
-          IsFileType(aFilename, sExecutableExts, ArrayLength(sExecutableExts)));
+              std::size(ApplicationReputationService::kNonBinaryExecutables)) &&
+          IsFileType(aFilename, sExecutableExts, std::size(sExecutableExts)));
 }
 
 ClientDownloadRequest::DownloadType PendingLookup::GetDownloadType(
@@ -1034,7 +1043,7 @@ nsresult PendingLookup::LookupNext() {
   // go ahead and pass.
   if (mAllowlistCount > 0) {
     return OnComplete(nsIApplicationReputationService::VERDICT_SAFE,
-                      Reason::LocalWhitelist, NS_OK);
+                      Reason::eLocalwhitelist, NS_OK);
   }
 
   MOZ_ASSERT_IF(!mIsBinaryFile, mAllowlistSpecs.Length() == 0);
@@ -1049,49 +1058,58 @@ nsresult PendingLookup::LookupNext() {
 
   if (!mFileName.IsEmpty()) {
     if (IsBinary(mFileName)) {
-      AccumulateCategorical(
-          mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_TYPE::
-              BinaryFile);
+      mozilla::glean::application_reputation::binary_type
+          .EnumGet(mozilla::glean::application_reputation::BinaryTypeLabel::
+                       eBinaryfile)
+          .Add();
     } else if (IsFileType(mFileName, kSafeFileExtensions,
-                          ArrayLength(kSafeFileExtensions))) {
-      AccumulateCategorical(
-          mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_TYPE::
-              NonBinaryFile);
+                          std::size(kSafeFileExtensions))) {
+      mozilla::glean::application_reputation::binary_type
+          .EnumGet(mozilla::glean::application_reputation::BinaryTypeLabel::
+                       eNonbinaryfile)
+          .Add();
     } else if (IsFileType(mFileName, kMozNonBinaryExecutables,
-                          ArrayLength(kMozNonBinaryExecutables))) {
-      AccumulateCategorical(
-          mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_TYPE::
-              MozNonBinaryFile);
+                          std::size(kMozNonBinaryExecutables))) {
+      mozilla::glean::application_reputation::binary_type
+          .EnumGet(mozilla::glean::application_reputation::BinaryTypeLabel::
+                       eMoznonbinaryfile)
+          .Add();
     } else {
-      AccumulateCategorical(
-          mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_TYPE::
-              UnknownFile);
+      mozilla::glean::application_reputation::binary_type
+          .EnumGet(mozilla::glean::application_reputation::BinaryTypeLabel::
+                       eUnknownfile)
+          .Add();
     }
   } else {
-    AccumulateCategorical(
-        mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_TYPE::
-            MissingFilename);
+    mozilla::glean::application_reputation::binary_type
+        .EnumGet(mozilla::glean::application_reputation::BinaryTypeLabel::
+                     eMissingfilename)
+        .Add();
   }
 
   if (IsFileType(mFileName, kDmgFileExtensions,
-                 ArrayLength(kDmgFileExtensions))) {
-    AccumulateCategorical(
-        mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_ARCHIVE::
-            DmgFile);
+                 std::size(kDmgFileExtensions))) {
+    mozilla::glean::application_reputation::binary_archive
+        .EnumGet(mozilla::glean::application_reputation::BinaryArchiveLabel::
+                     eDmgfile)
+        .Add();
   } else if (IsFileType(mFileName, kRarFileExtensions,
-                        ArrayLength(kRarFileExtensions))) {
-    AccumulateCategorical(
-        mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_ARCHIVE::
-            RarFile);
+                        std::size(kRarFileExtensions))) {
+    mozilla::glean::application_reputation::binary_archive
+        .EnumGet(mozilla::glean::application_reputation::BinaryArchiveLabel::
+                     eRarfile)
+        .Add();
   } else if (IsFileType(mFileName, kZipFileExtensions,
-                        ArrayLength(kZipFileExtensions))) {
-    AccumulateCategorical(
-        mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_ARCHIVE::
-            ZipFile);
+                        std::size(kZipFileExtensions))) {
+    mozilla::glean::application_reputation::binary_archive
+        .EnumGet(mozilla::glean::application_reputation::BinaryArchiveLabel::
+                     eZipfile)
+        .Add();
   } else if (mIsBinaryFile) {
-    AccumulateCategorical(
-        mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_BINARY_ARCHIVE::
-            OtherBinaryFile);
+    mozilla::glean::application_reputation::binary_archive
+        .EnumGet(mozilla::glean::application_reputation::BinaryArchiveLabel::
+                     eOtherbinaryfile)
+        .Add();
   }
 
   // There are no more URIs to check against local list. If the file is
@@ -1099,13 +1117,13 @@ nsresult PendingLookup::LookupNext() {
   if (!mIsBinaryFile) {
     LOG(("Not eligible for remote lookups [this=%p]", this));
     return OnComplete(nsIApplicationReputationService::VERDICT_SAFE,
-                      Reason::NonBinaryFile, NS_OK);
+                      Reason::eNonbinaryfile, NS_OK);
   }
 
   nsresult rv = SendRemoteQuery();
   if (NS_FAILED(rv)) {
     return OnComplete(nsIApplicationReputationService::VERDICT_SAFE,
-                      Reason::InternalError, rv);
+                      Reason::eInternalerror, rv);
   }
   return NS_OK;
 }
@@ -1287,19 +1305,16 @@ nsresult PendingLookup::StartLookup() {
   nsresult rv = DoLookupInternal();
   if (NS_FAILED(rv)) {
     return OnComplete(nsIApplicationReputationService::VERDICT_SAFE,
-                      Reason::InternalError, NS_OK);
+                      Reason::eInternalerror, NS_OK);
   }
   return rv;
 }
 
 nsresult PendingLookup::GetSpecHash(nsACString& aSpec,
                                     nsACString& hexEncodedHash) {
-  nsresult rv;
-
-  nsCOMPtr<nsICryptoHash> cryptoHash =
-      do_CreateInstance("@mozilla.org/security/hash;1", &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = cryptoHash->Init(nsICryptoHash::SHA256);
+  nsCOMPtr<nsICryptoHash> cryptoHash;
+  nsresult rv =
+      NS_NewCryptoHash(nsICryptoHash::SHA256, getter_AddRefs(cryptoHash));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = cryptoHash->Update(
@@ -1489,25 +1504,25 @@ nsresult PendingLookup::OnComplete(uint32_t aVerdict, Reason aReason,
     case nsIApplicationReputationService::VERDICT_DANGEROUS:
       if (!Preferences::GetBool(PREF_BLOCK_DANGEROUS, true)) {
         shouldBlock = false;
-        aReason = Reason::DangerousPrefOff;
+        aReason = Reason::eDangerousprefoff;
       }
       break;
     case nsIApplicationReputationService::VERDICT_UNCOMMON:
       if (!Preferences::GetBool(PREF_BLOCK_UNCOMMON, true)) {
         shouldBlock = false;
-        aReason = Reason::UncommonPrefOff;
+        aReason = Reason::eUncommonprefoff;
       }
       break;
     case nsIApplicationReputationService::VERDICT_POTENTIALLY_UNWANTED:
       if (!Preferences::GetBool(PREF_BLOCK_POTENTIALLY_UNWANTED, true)) {
         shouldBlock = false;
-        aReason = Reason::UnwantedPrefOff;
+        aReason = Reason::eUnwantedprefoff;
       }
       break;
     case nsIApplicationReputationService::VERDICT_DANGEROUS_HOST:
       if (!Preferences::GetBool(PREF_BLOCK_DANGEROUS_HOST, true)) {
         shouldBlock = false;
-        aReason = Reason::DangerousHostPrefOff;
+        aReason = Reason::eDangeroushostprefoff;
       }
       break;
     default:
@@ -1515,9 +1530,12 @@ nsresult PendingLookup::OnComplete(uint32_t aVerdict, Reason aReason,
       break;
   }
 
-  AccumulateCategorical(aReason);
-  Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SHOULD_BLOCK,
-             shouldBlock);
+  mozilla::glean::application_reputation::reason.EnumGet(aReason).Add();
+  mozilla::glean::application_reputation::should_block
+      .EnumGet(
+          static_cast<mozilla::glean::application_reputation::ShouldBlockLabel>(
+              shouldBlock))
+      .Add();
 
   double t = (TimeStamp::Now() - mStartTime).ToMilliseconds();
   LOG(("Application Reputation verdict is %u, obtained in %f ms [this = %p]",
@@ -1557,8 +1575,8 @@ nsresult PendingLookup::ParseCertificates(
 nsresult PendingLookup::SendRemoteQuery() {
   MOZ_ASSERT(!IsFileType(
       mFileName, ApplicationReputationService::kNonBinaryExecutables,
-      ArrayLength(ApplicationReputationService::kNonBinaryExecutables)));
-  Reason reason = Reason::NotSet;
+      std::size(ApplicationReputationService::kNonBinaryExecutables)));
+  Reason reason = Reason::eNotset;
   nsresult rv = SendRemoteQueryInternal(reason);
   if (NS_FAILED(rv)) {
     return OnComplete(nsIApplicationReputationService::VERDICT_SAFE, reason,
@@ -1571,15 +1589,15 @@ nsresult PendingLookup::SendRemoteQuery() {
 
 nsresult PendingLookup::SendRemoteQueryInternal(Reason& aReason) {
   auto scopeExit = mozilla::MakeScopeExit([&aReason]() {
-    if (aReason == Reason::NotSet) {
-      aReason = Reason::InternalError;
+    if (aReason == Reason::eNotset) {
+      aReason = Reason::eInternalerror;
     }
   });
 
   // If we aren't supposed to do remote lookups, bail.
   if (!Preferences::GetBool(PREF_SB_DOWNLOADS_REMOTE_ENABLED, false)) {
     LOG(("Remote lookups are disabled [this = %p]", this));
-    aReason = Reason::RemoteLookupDisabled;
+    aReason = Reason::eRemotelookupdisabled;
     return NS_ERROR_NOT_AVAILABLE;
   }
   // If the remote lookup URL is empty or absent, bail.
@@ -1591,7 +1609,7 @@ nsresult PendingLookup::SendRemoteQueryInternal(Reason& aReason) {
           NS_ConvertASCIItoUTF16(PREF_SB_APP_REP_URL), serviceUrl)) ||
       serviceUrl.IsEmpty() || u"about:blank"_ns.Equals(serviceUrl)) {
     LOG(("Remote lookup URL is empty or absent [this = %p]", this));
-    aReason = Reason::RemoteLookupDisabled;
+    aReason = Reason::eRemotelookupdisabled;
     return NS_ERROR_NOT_AVAILABLE;
   }
 
@@ -1660,7 +1678,7 @@ nsresult PendingLookup::SendRemoteQueryInternal(Reason& aReason) {
       do_CreateInstance("@mozilla.org/io/string-input-stream;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = sstream->SetData(serialized.c_str(), serialized.length());
+  rv = sstream->CopyData(serialized.c_str(), serialized.length());
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Set up the channel to transmit the request to the service.
@@ -1710,8 +1728,10 @@ NS_IMETHODIMP
 PendingLookup::Notify(nsITimer* aTimer) {
   LOG(("Remote lookup timed out [this = %p]", this));
   MOZ_ASSERT(aTimer == mTimeoutTimer);
-  Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_REMOTE_LOOKUP_TIMEOUT,
-             true);
+  mozilla::glean::application_reputation::remote_lookup_timeout
+      .EnumGet(mozilla::glean::application_reputation::
+                   RemoteLookupTimeoutLabel::eTrue)
+      .Add();
   mChannel->Cancel(NS_ERROR_NET_TIMEOUT_EXTERNAL);
   mTimeoutTimer->Cancel();
   return NS_OK;
@@ -1767,21 +1787,22 @@ PendingLookup::OnStopRequest(nsIRequest* aRequest, nsresult aResult) {
   NS_ENSURE_STATE(mCallback);
 
   if (aResult != NS_ERROR_NET_TIMEOUT_EXTERNAL) {
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_REMOTE_LOOKUP_TIMEOUT,
-               false);
+    mozilla::glean::application_reputation::remote_lookup_timeout
+        .EnumGet(mozilla::glean::application_reputation::
+                     RemoteLookupTimeoutLabel::eFalse)
+        .Add();
 
     MOZ_ASSERT(mTelemetryRemoteRequestStartMs > 0);
     int32_t msecs = PR_IntervalToMilliseconds(PR_IntervalNow() -
                                               mTelemetryRemoteRequestStartMs);
 
     MOZ_ASSERT(msecs >= 0);
-    mozilla::Telemetry::Accumulate(
-        mozilla::Telemetry::APPLICATION_REPUTATION_REMOTE_LOOKUP_RESPONSE_TIME,
-        msecs);
+    mozilla::glean::application_reputation::remote_lookup_response_time
+        .AccumulateRawDuration(mozilla::TimeDuration::FromMilliseconds(msecs));
   }
 
   uint32_t verdict = nsIApplicationReputationService::VERDICT_SAFE;
-  Reason reason = Reason::NotSet;
+  Reason reason = Reason::eNotset;
   nsresult rv = OnStopRequestInternal(aRequest, aResult, verdict, reason);
   OnComplete(verdict, reason, rv);
   return rv;
@@ -1793,44 +1814,50 @@ nsresult PendingLookup::OnStopRequestInternal(nsIRequest* aRequest,
                                               Reason& aReason) {
   auto scopeExit = mozilla::MakeScopeExit([&aReason]() {
     // If |aReason| is not set while exiting, there must be an error.
-    if (aReason == Reason::NotSet) {
-      aReason = Reason::NetworkError;
+    if (aReason == Reason::eNotset) {
+      aReason = Reason::eNetworkerror;
     }
   });
 
   if (NS_FAILED(aResult)) {
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SERVER,
-               SERVER_RESPONSE_FAILED);
-    AccumulateCategorical(NSErrorToLabel(aResult));
+    mozilla::glean::application_reputation::server.AccumulateSingleSample(
+        SERVER_RESPONSE_FAILED);
+    mozilla::glean::application_reputation::server_2
+        .EnumGet(NSErrorToLabel(aResult))
+        .Add();
     return aResult;
   }
 
   nsresult rv;
   nsCOMPtr<nsIHttpChannel> channel = do_QueryInterface(aRequest, &rv);
   if (NS_FAILED(rv)) {
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SERVER,
-               SERVER_RESPONSE_FAILED);
-    AccumulateCategorical(
-        mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_SERVER_2::
-            FailGetChannel);
+    mozilla::glean::application_reputation::server.AccumulateSingleSample(
+        SERVER_RESPONSE_FAILED);
+    mozilla::glean::application_reputation::server_2
+        .EnumGet(mozilla::glean::application_reputation::Server2Label::
+                     eFailgetchannel)
+        .Add();
     return rv;
   }
 
   uint32_t status = 0;
   rv = channel->GetResponseStatus(&status);
   if (NS_FAILED(rv)) {
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SERVER,
-               SERVER_RESPONSE_FAILED);
-    AccumulateCategorical(
-        mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_SERVER_2::
-            FailGetResponse);
+    mozilla::glean::application_reputation::server.AccumulateSingleSample(
+        SERVER_RESPONSE_FAILED);
+    mozilla::glean::application_reputation::server_2
+        .EnumGet(mozilla::glean::application_reputation::Server2Label::
+                     eFailgetresponse)
+        .Add();
     return rv;
   }
 
   if (status != 200) {
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SERVER,
-               SERVER_RESPONSE_FAILED);
-    AccumulateCategorical(HTTPStatusToLabel(status));
+    mozilla::glean::application_reputation::server.AccumulateSingleSample(
+        SERVER_RESPONSE_FAILED);
+    mozilla::glean::application_reputation::server_2
+        .EnumGet(HTTPStatusToLabel(status))
+        .Add();
     return NS_ERROR_NOT_AVAILABLE;
   }
 
@@ -1839,52 +1866,53 @@ nsresult PendingLookup::OnStopRequestInternal(nsIRequest* aRequest,
   if (!response.ParseFromString(buf)) {
     LOG(("Invalid protocol buffer response [this = %p]: %s", this,
          buf.c_str()));
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SERVER,
-               SERVER_RESPONSE_INVALID);
+    mozilla::glean::application_reputation::server.AccumulateSingleSample(
+        SERVER_RESPONSE_INVALID);
     return NS_ERROR_CANNOT_CONVERT_DATA;
   }
 
-  Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SERVER,
-             SERVER_RESPONSE_VALID);
-  AccumulateCategorical(
-      mozilla::Telemetry::LABELS_APPLICATION_REPUTATION_SERVER_2::
-          ResponseValid);
+  mozilla::glean::application_reputation::server.AccumulateSingleSample(
+      SERVER_RESPONSE_VALID);
+  mozilla::glean::application_reputation::server_2
+      .EnumGet(
+          mozilla::glean::application_reputation::Server2Label::eResponsevalid)
+      .Add();
 
   // Clamp responses 0-7, we only know about 0-4 for now.
-  Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SERVER_VERDICT,
-             std::min<uint32_t>(response.verdict(), 7));
+  mozilla::glean::application_reputation::server_verdict.AccumulateSingleSample(
+      std::min<uint32_t>(response.verdict(), 7));
   const char* ext = GetFileExt(mFileName);
   AccumulateCategoricalKeyed(nsCString(ext), VerdictToLabel(std::min<uint32_t>(
                                                  response.verdict(), 7)));
   switch (response.verdict()) {
     case safe_browsing::ClientDownloadResponse::DANGEROUS:
       aVerdict = nsIApplicationReputationService::VERDICT_DANGEROUS;
-      aReason = Reason::VerdictDangerous;
+      aReason = Reason::eVerdictdangerous;
       break;
     case safe_browsing::ClientDownloadResponse::DANGEROUS_HOST:
       aVerdict = nsIApplicationReputationService::VERDICT_DANGEROUS_HOST;
-      aReason = Reason::VerdictDangerousHost;
+      aReason = Reason::eVerdictdangeroushost;
       break;
     case safe_browsing::ClientDownloadResponse::POTENTIALLY_UNWANTED:
       aVerdict = nsIApplicationReputationService::VERDICT_POTENTIALLY_UNWANTED;
-      aReason = Reason::VerdictUnwanted;
+      aReason = Reason::eVerdictunwanted;
       break;
     case safe_browsing::ClientDownloadResponse::UNCOMMON:
       aVerdict = nsIApplicationReputationService::VERDICT_UNCOMMON;
-      aReason = Reason::VerdictUncommon;
+      aReason = Reason::eVerdictuncommon;
       break;
     case safe_browsing::ClientDownloadResponse::UNKNOWN:
       aVerdict = nsIApplicationReputationService::VERDICT_SAFE;
-      aReason = Reason::VerdictUnknown;
+      aReason = Reason::eVerdictunknown;
       break;
     case safe_browsing::ClientDownloadResponse::SAFE:
       aVerdict = nsIApplicationReputationService::VERDICT_SAFE;
-      aReason = Reason::VerdictSafe;
+      aReason = Reason::eVerdictsafe;
       break;
     default:
       // Treat everything else as safe
       aVerdict = nsIApplicationReputationService::VERDICT_SAFE;
-      aReason = Reason::VerdictUnrecognized;
+      aReason = Reason::eVerdictunrecognized;
       break;
   }
 
@@ -1925,11 +1953,14 @@ ApplicationReputationService::QueryReputation(
 
   nsresult rv = QueryReputationInternal(aQuery, aCallback);
   if (NS_FAILED(rv)) {
-    Reason reason = rv == NS_ERROR_NOT_AVAILABLE ? Reason::DPDisabled
-                                                 : Reason::InternalError;
+    Reason reason = rv == NS_ERROR_NOT_AVAILABLE ? Reason::eDpdisabled
+                                                 : Reason::eInternalerror;
 
-    AccumulateCategorical(reason);
-    Accumulate(mozilla::Telemetry::APPLICATION_REPUTATION_SHOULD_BLOCK, false);
+    mozilla::glean::application_reputation::reason.EnumGet(reason).Add();
+    mozilla::glean::application_reputation::should_block
+        .EnumGet(
+            mozilla::glean::application_reputation::ShouldBlockLabel::eFalse)
+        .Add();
 
     aCallback->OnComplete(false, rv,
                           nsIApplicationReputationService::VERDICT_SAFE);
@@ -1978,6 +2009,6 @@ nsresult ApplicationReputationService::IsBinary(const nsACString& aFileName,
 nsresult ApplicationReputationService::IsExecutable(const nsACString& aFileName,
                                                     bool* aExecutable) {
   *aExecutable =
-      ::IsFileType(aFileName, sExecutableExts, ArrayLength(sExecutableExts));
+      ::IsFileType(aFileName, sExecutableExts, std::size(sExecutableExts));
   return NS_OK;
 }
