@@ -1,12 +1,11 @@
-add_task(async function() {
+add_task(async function () {
   await BrowserTestUtils.withNewTab(
     {
       gBrowser,
-      url:
-        "data:text/html,<!DOCTYPE html><html><body><input id='i'></body></html>",
+      url: "data:text/html,<!DOCTYPE html><html><body><input id='i'></body></html>",
     },
-    async function(browser) {
-      await ContentTask.spawn(browser, "", function() {
+    async function (browser) {
+      await SpecialPowers.spawn(browser, [""], function () {
         let tttp = Cc[
           "@mozilla.org/embedcomp/default-tooltiptextprovider;1"
         ].getService(Ci.nsITooltipTextProvider);
