@@ -2,6 +2,18 @@
  * http://creativecommons.org/publicdomain/zero/1.0/
  */
 
+// TODO bug 1649906: strip blocklist v2-specific parts of this test.
+const useMLBF = Services.prefs.getBoolPref(
+  "extensions.blocklist.useMLBF",
+  true
+);
+
+// Enable soft-blocks support on MLBF blocklist by default while running
+// this test file.
+if (useMLBF) {
+  Services.prefs.setBoolPref("extensions.blocklist.softblock.enabled", true);
+}
+
 // Tests that an appDisabled add-on that becomes softBlocked remains disabled
 // when becoming appEnabled
 add_task(async function test_softblock() {
@@ -12,7 +24,7 @@ add_task(async function test_softblock() {
     manifest: {
       name: "Softblocked add-on",
       version: "1.0",
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: "softblock1@tests.mozilla.org",
           strict_min_version: "2",
@@ -31,6 +43,18 @@ add_task(async function test_softblock() {
   Assert.ok(!s1.isActive);
 
   await AddonTestUtils.loadBlocklistRawData({
+    extensionsMLBF: [
+      {
+        stash: {
+          softblocked: ["softblock1@tests.mozilla.org:1.0"],
+          blocked: [],
+          unblocked: [],
+        },
+        stash_time: Date.now(),
+      },
+    ],
+    // TODO: (Bug 1649906) remove this blocklist test data along with
+    // removing Blocklist v2 implementation.
     extensions: [
       {
         guid: "softblock1@tests.mozilla.org",
