@@ -2,7 +2,7 @@
  * http://creativecommons.org/publicdomain/zero/1.0/
  */
 
-// Tests caching in AddonRepository.jsm
+// Tests caching in AddonRepository.sys.mjs.
 
 var gServer;
 
@@ -37,7 +37,7 @@ const ADDONS = [
         page: "options.html",
       },
 
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "test_AddonRepository_1@tests.mozilla.org" },
       },
     },
@@ -47,7 +47,7 @@ const ADDONS = [
       name: "XPI Add-on 2",
       version: "1.2",
       theme: {},
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "test_AddonRepository_2@tests.mozilla.org" },
       },
     },
@@ -60,7 +60,7 @@ const ADDONS = [
         32: "icon.png",
       },
       theme: {},
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "test_AddonRepository_3@tests.mozilla.org" },
       },
     },
@@ -70,7 +70,9 @@ const ADDONS = [
   },
 ];
 
-const ADDON_IDS = ADDONS.map(addon => addon.manifest.applications.gecko.id);
+const ADDON_IDS = ADDONS.map(
+  addon => addon.manifest.browser_specific_settings.gecko.id
+);
 const ADDON_FILES = ADDONS.map(addon =>
   AddonTestUtils.createTempWebExtensionFile(addon)
 );
@@ -135,7 +137,7 @@ const REPOSITORY_ADDONS = [
     ],
     description: "Repo Add-on 1 - Description\nSecond line",
     fullDescription: "Repo Add-on 1 - Full Description & some extra",
-    icons: { "32": BASE_URL + "/repo/1/icon.png" },
+    icons: { 32: BASE_URL + "/repo/1/icon.png" },
     homepageURL: BASE_URL + "/repo/1/homepage.html",
     supportURL: BASE_URL + "/repo/1/support.html",
     averageRating: 1,
@@ -161,7 +163,7 @@ const REPOSITORY_ADDONS = [
     ],
     description: "Repo Add-on 2 - Description",
     fullDescription: "Repo Add-on 2 - Full Description",
-    icons: { "32": BASE_URL + "/repo/2/icon.png" },
+    icons: { 32: BASE_URL + "/repo/2/icon.png" },
     screenshots: [
       {
         url: BASE_URL + "/repo/2/firstFull.png",
@@ -187,7 +189,7 @@ const REPOSITORY_ADDONS = [
     type: "theme",
     name: "Repo Add-on 3",
     version: "2.3",
-    icons: { "32": BASE_URL + "/repo/3/icon.png" },
+    icons: { 32: BASE_URL + "/repo/3/icon.png" },
     screenshots: [
       {
         url: BASE_URL + "/repo/3/firstFull.png",
@@ -217,7 +219,7 @@ const WITHOUT_CACHE = [
     authors: [{ name: "XPI Add-on 1 - Author" }],
     description: "XPI Add-on 1 - Description",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[0], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[0], "icon.png") };
     },
     homepageURL: `${BASE_URL}/xpi/1/homepage.html`,
     get optionsURL() {
@@ -239,7 +241,7 @@ const WITHOUT_CACHE = [
     name: "XPI Add-on 3",
     version: "1.3",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[2], "icon.png") };
     },
     screenshots: [
       {
@@ -272,7 +274,7 @@ const WITH_CACHE = [
     description: "XPI Add-on 1 - Description",
     fullDescription: "Repo Add-on 1 - Full Description & some extra",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[0], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[0], "icon.png") };
     },
     homepageURL: BASE_URL + "/xpi/1/homepage.html",
     supportURL: BASE_URL + "/repo/1/support.html",
@@ -302,7 +304,7 @@ const WITH_CACHE = [
     ],
     description: "Repo Add-on 2 - Description",
     fullDescription: "Repo Add-on 2 - Full Description",
-    icons: { "32": BASE_URL + "/repo/2/icon.png" },
+    icons: { 32: BASE_URL + "/repo/2/icon.png" },
     screenshots: [
       {
         url: BASE_URL + "/repo/2/firstFull.png",
@@ -332,7 +334,7 @@ const WITH_CACHE = [
       return get_subfile_uri(ADDON_IDS[2], "icon.png");
     },
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[2], "icon.png") };
     },
     screenshots: [
       {
@@ -350,7 +352,7 @@ const WITH_CACHE = [
   },
 ];
 
-// Expected add-ons when using cache
+// Expected add-ons when using cache for extension, but no cache for themes.
 const WITH_EXTENSION_CACHE = [
   {
     id: ADDON_IDS[0],
@@ -370,7 +372,7 @@ const WITH_EXTENSION_CACHE = [
     description: "XPI Add-on 1 - Description",
     fullDescription: "Repo Add-on 1 - Full Description & some extra",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[0], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[0], "icon.png") };
     },
     homepageURL: BASE_URL + "/xpi/1/homepage.html",
     supportURL: BASE_URL + "/repo/1/support.html",
@@ -400,7 +402,7 @@ const WITH_EXTENSION_CACHE = [
       return get_subfile_uri(ADDON_IDS[2], "icon.png");
     },
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[2], "icon.png") };
     },
     screenshots: [
       {
@@ -434,7 +436,7 @@ function check_results(aActualAddons, aExpectedAddons, aFromRepository) {
 
   // Separately test updateDate (it should only be equal to the
   // REPOSITORY values if it is from the repository)
-  aActualAddons.forEach(function(aActualAddon) {
+  aActualAddons.forEach(function (aActualAddon) {
     if (aActualAddon.updateDate) {
       let time = aActualAddon.updateDate.getTime();
       Assert.equal(time === 1000 * REPOSITORY_UPDATEDATE, aFromRepository);
@@ -463,22 +465,23 @@ function check_cache(aExpectedToFind, aExpectedImmediately) {
 
   for (let i = 0; i < REPOSITORY_ADDONS.length; i++) {
     lookups.push(
-      new Promise((resolve, reject) => {
+      new Promise(resolve => {
         let immediatelyFound = true;
         let expected = aExpectedToFind[i] ? REPOSITORY_ADDONS[i] : null;
         // can't Promise-wrap this because we're also testing whether the callback is
         // sync or async
-        AddonRepository.getCachedAddonByID(REPOSITORY_ADDONS[i].id, function(
-          aAddon
-        ) {
-          Assert.equal(immediatelyFound, aExpectedImmediately);
-          if (expected == null) {
-            Assert.equal(aAddon, null);
-          } else {
-            check_results([aAddon], [expected], true);
+        AddonRepository.getCachedAddonByID(
+          REPOSITORY_ADDONS[i].id,
+          function (aAddon) {
+            Assert.equal(immediatelyFound, aExpectedImmediately);
+            if (expected == null) {
+              Assert.equal(aAddon, null);
+            } else {
+              check_results([aAddon], [expected], true);
+            }
+            resolve();
           }
-          resolve();
-        });
+        );
         immediatelyFound = false;
       })
     );
@@ -708,4 +711,18 @@ add_task(async function run_test_17() {
   await AddonManagerPrivate.backgroundUpdateCheck();
   let aAddons = await promiseAddonsByIDs(ADDON_IDS);
   check_results(aAddons, WITH_EXTENSION_CACHE);
+  Services.prefs.clearUserPref(PREF_GETADDONS_CACHE_TYPES);
+});
+
+// Tests that the cache is retained when the server/API is unreachable.
+// Regression test for https://bugzilla.mozilla.org/show_bug.cgi?id=1870905
+add_task(async function run_test_18() {
+  // The response is expected to be JSON, so setting it to non-JSON is
+  // equivalent to the server being unreachable in the implementation.
+  Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, "data:text/not-json,");
+
+  await AddonManagerPrivate.backgroundUpdateCheck();
+  let aAddons = await promiseAddonsByIDs(ADDON_IDS);
+  check_results(aAddons, WITH_EXTENSION_CACHE);
+  Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, GETADDONS_RESULTS);
 });
