@@ -2,14 +2,24 @@
 // Test whether setting a new property in InstallTrigger then persists to other
 // page loads
 add_task(async function test() {
+  await SpecialPowers.pushPrefEnv({
+    set: [
+      ["extensions.InstallTrigger.enabled", true],
+      ["extensions.InstallTriggerImpl.enabled", true],
+    ],
+  });
+
   await BrowserTestUtils.withNewTab(
     { gBrowser, url: TESTROOT + "enabled.html" },
-    async function(browser) {
+    async function (browser) {
       await SpecialPowers.spawn(browser, [], () => {
-        content.wrappedJSObject.InstallTrigger.enabled.k = function() {};
+        content.wrappedJSObject.InstallTrigger.enabled.k = function () {};
       });
 
-      BrowserTestUtils.loadURI(browser, TESTROOT2 + "enabled.html");
+      BrowserTestUtils.startLoadingURIString(
+        browser,
+        TESTROOT2 + "enabled.html"
+      );
       await BrowserTestUtils.browserLoaded(browser);
       await SpecialPowers.spawn(browser, [], () => {
         is(
