@@ -35,7 +35,6 @@
 #include "mozilla/Logging.h"
 #include "mozilla/MiscEvents.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/TextEvents.h"
 #include "mozilla/StaticMutex.h"
 #include "mozilla/StaticPrefs_media.h"
@@ -295,7 +294,7 @@ BOOL nsCocoaUtils::WasLaunchedAtLogin() {
   return NO;
 }
 
-BOOL nsCocoaUtils::ShouldRestoreStateDueToLaunchAtLoginImpl() {
+BOOL nsCocoaUtils::ShouldRestoreStateDueToLaunchAtLogin() {
   // Check if we were launched by macOS as a result of having
   // "Reopen windows..." selected during a restart.
   if (!WasLaunchedAtLogin()) {
@@ -304,8 +303,8 @@ BOOL nsCocoaUtils::ShouldRestoreStateDueToLaunchAtLoginImpl() {
 
   CFStringRef lgnwPlistName = CFSTR("com.apple.loginwindow");
   CFStringRef saveStateKey = CFSTR("TALLogoutSavesState");
-  CFPropertyListRef lgnwPlist = (CFPropertyListRef)(::CFPreferencesCopyAppValue(
-      saveStateKey, lgnwPlistName));
+  auto lgnwPlist = CFTypeRefPtr<CFPropertyListRef>::WrapUnderCreateRule(
+      ::CFPreferencesCopyAppValue(saveStateKey, lgnwPlistName));
   // The .plist doesn't exist unless the user changed the "Reopen windows..."
   // preference. If it doesn't exist, restore by default (as this is the macOS
   // default).
@@ -314,23 +313,28 @@ BOOL nsCocoaUtils::ShouldRestoreStateDueToLaunchAtLoginImpl() {
     return YES;
   }
 
-  if (CFBooleanRef shouldRestoreState = static_cast<CFBooleanRef>(lgnwPlist)) {
+  if (::CFGetTypeID(lgnwPlist.get()) != ::CFBooleanGetTypeID()) {
+    return YES;
+  }
+
+  if (CFBooleanRef shouldRestoreState =
+          static_cast<CFBooleanRef>(lgnwPlist.get())) {
     return ::CFBooleanGetValue(shouldRestoreState);
   }
 
   return NO;
 }
 
-BOOL nsCocoaUtils::ShouldRestoreStateDueToLaunchAtLogin() {
-  BOOL shouldRestore = ShouldRestoreStateDueToLaunchAtLoginImpl();
-  Telemetry::ScalarSet(Telemetry::ScalarID::STARTUP_IS_RESTORED_BY_MACOS,
-                       !!shouldRestore);
-  return shouldRestore;
-}
-
 void nsCocoaUtils::PrepareForNativeAppModalDialog() {
   NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
 
+  if (!NSApp.active) {
+    // Early exit if the app isn't active. This is because we can't safely
+    // set the NSApp.mainMenu property in such a case. We early exit so we
+    // also don't invoke any side effects.
+    return;
+  }
+
   // Don't do anything if this is embedding. We'll assume that if there is no
   // hidden window we shouldn't do anything, and that should cover the embedding
   // case.
@@ -376,10 +380,13 @@ void nsCocoaUtils::CleanUpAfterNativeAppModalDialog() {
   if (!hiddenWindowMenuBar) return;
 
   NSWindow* mainWindow = [NSApp mainWindow];
-  if (!mainWindow)
-    hiddenWindowMenuBar->Paint();
-  else
+  if (!mainWindow) {
+    // We do an async paint in order to prevent crashes when macOS is actively
+    // enumerating the menu items in `NSApp.mainMenu`.
+    hiddenWindowMenuBar->PaintAsyncIfNeeded();
+  } else {
     [WindowDelegate paintMenubarForWindow:mainWindow];
+  }
 
   NS_OBJC_END_TRY_IGNORE_BLOCK;
 }
@@ -514,25 +521,24 @@ nsresult nsCocoaUtils::CreateNSImageFromCGImage(CGImageRef aInputImage,
 
 nsresult nsCocoaUtils::CreateNSImageFromImageContainer(
     imgIContainer* aImage, uint32_t aWhichFrame,
-    const nsPresContext* aPresContext, const ComputedStyle* aComputedStyle,
-    const NSSize& aPreferredSize, NSImage** aResult, CGFloat scaleFactor,
-    bool* aIsEntirelyBlack) {
+    const SVGImageContext* aSVGContext, const NSSize& aPreferredSize,
+    NSImage** aResult, CGFloat scaleFactor, bool* aIsEntirelyBlack) {
   RefPtr<SourceSurface> surface;
   int32_t width = 0;
   int32_t height = 0;
   {
     const bool gotWidth = NS_SUCCEEDED(aImage->GetWidth(&width));
     const bool gotHeight = NS_SUCCEEDED(aImage->GetHeight(&height));
-    if (auto ratio = aImage->GetIntrinsicRatio(); ratio && *ratio) {
+    if (auto ratio = aImage->GetIntrinsicRatio()) {
       if (gotWidth != gotHeight) {
         if (gotWidth) {
-          height = ratio->Inverted().ApplyTo(width);
+          height = ratio.Inverted().ApplyTo(width);
         } else {
-          width = ratio->ApplyTo(height);
+          width = ratio.ApplyTo(height);
         }
       } else if (!gotWidth) {
         height = std::ceil(aPreferredSize.height);
-        width = ratio->ApplyTo(height);
+        width = ratio.ApplyTo(height);
       }
     }
   }
@@ -552,14 +558,15 @@ nsresult nsCocoaUtils::CreateNSImageFromImageContainer(
 
     gfxContext context(drawTarget);
 
-    SVGImageContext svgContext;
-    if (aPresContext && aComputedStyle) {
-      SVGImageContext::MaybeStoreContextPaint(svgContext, *aPresContext,
-                                              *aComputedStyle, aImage);
+    UniquePtr<SVGImageContext> svgContext;
+    if (!aSVGContext) {
+      svgContext = MakeUnique<SVGImageContext>();
+      aSVGContext = svgContext.get();
     }
+
     mozilla::image::ImgDrawResult res =
         aImage->Draw(&context, scaledSize, ImageRegion::Create(scaledSize),
-                     aWhichFrame, SamplingFilter::POINT, svgContext,
+                     aWhichFrame, SamplingFilter::POINT, *aSVGContext,
                      imgIContainer::FLAG_SYNC_DECODE, 1.0);
 
     if (res != mozilla::image::ImgDrawResult::SUCCESS) {
@@ -597,12 +604,12 @@ nsresult nsCocoaUtils::CreateNSImageFromImageContainer(
 
 nsresult nsCocoaUtils::CreateDualRepresentationNSImageFromImageContainer(
     imgIContainer* aImage, uint32_t aWhichFrame,
-    const nsPresContext* aPresContext, const ComputedStyle* aComputedStyle,
-    const NSSize& aPreferredSize, NSImage** aResult, bool* aIsEntirelyBlack) {
+    const SVGImageContext* aSVGContext, const NSSize& aPreferredSize,
+    NSImage** aResult, bool* aIsEntirelyBlack) {
   NSImage* newRepresentation = nil;
   nsresult rv = CreateNSImageFromImageContainer(
-      aImage, aWhichFrame, aPresContext, aComputedStyle, aPreferredSize,
-      &newRepresentation, 1.0f, aIsEntirelyBlack);
+      aImage, aWhichFrame, aSVGContext, aPreferredSize, &newRepresentation,
+      1.0f, aIsEntirelyBlack);
   if (NS_FAILED(rv) || !newRepresentation) {
     return NS_ERROR_FAILURE;
   }
@@ -617,9 +624,9 @@ nsresult nsCocoaUtils::CreateDualRepresentationNSImageFromImageContainer(
   [newRepresentation release];
   newRepresentation = nil;
 
-  rv = CreateNSImageFromImageContainer(
-      aImage, aWhichFrame, aPresContext, aComputedStyle, aPreferredSize,
-      &newRepresentation, 2.0f, aIsEntirelyBlack);
+  rv = CreateNSImageFromImageContainer(aImage, aWhichFrame, aSVGContext,
+                                       aPreferredSize, &newRepresentation, 2.0f,
+                                       aIsEntirelyBlack);
   if (NS_FAILED(rv) || !newRepresentation) {
     return NS_ERROR_FAILURE;
   }
@@ -713,7 +720,7 @@ NSEvent* nsCocoaUtils::MakeNewCococaEventFromWidgetEvent(
       {MODIFIER_FN, NSEventModifierFlagFunction}};
 
   NSUInteger modifierFlags = 0;
-  for (uint32_t i = 0; i < ArrayLength(sModifierFlagMap); ++i) {
+  for (uint32_t i = 0; i < std::size(sModifierFlagMap); ++i) {
     if (aKeyEvent.mModifiers & sModifierFlagMap[i][0]) {
       modifierFlags |= sModifierFlagMap[i][1];
     }
@@ -801,8 +808,9 @@ Modifiers nsCocoaUtils::ModifiersForEvent(NSEvent* aNativeEvent) {
   // that the event's keyCode falls outside the range of keys that will also set
   // the function modifier.
   if (!!(modifiers & NSEventModifierFlagFunction) &&
-      (aNativeEvent.type == NSKeyDown || aNativeEvent.type == NSKeyUp ||
-       aNativeEvent.type == NSFlagsChanged) &&
+      (aNativeEvent.type == NSEventTypeKeyDown ||
+       aNativeEvent.type == NSEventTypeKeyUp ||
+       aNativeEvent.type == NSEventTypeFlagsChanged) &&
       !(kVK_Return <= aNativeEvent.keyCode &&
         aNativeEvent.keyCode <= NSModeSwitchFunctionKey)) {
     result |= MODIFIER_FN;
@@ -1083,7 +1091,7 @@ uint32_t nsCocoaUtils::ConvertGeckoNameToMacCharCode(
 
   uint32_t keyCodeNameLength = keyCodeName.Length();
   const char* keyCodeNameStr = keyCodeName.get();
-  for (uint16_t i = 0; i < ArrayLength(gKeyConversions); ++i) {
+  for (uint16_t i = 0; i < std::size(gKeyConversions); ++i) {
     if (keyCodeNameLength == gKeyConversions[i].strLength &&
         nsCRT::strcmp(gKeyConversions[i].str, keyCodeNameStr) == 0) {
       return gKeyConversions[i].charCode;
@@ -1098,7 +1106,7 @@ uint32_t nsCocoaUtils::ConvertGeckoKeyCodeToMacCharCode(uint32_t aKeyCode) {
     return 0;
   }
 
-  for (uint16_t i = 0; i < ArrayLength(gKeyConversions); ++i) {
+  for (uint16_t i = 0; i < std::size(gKeyConversions); ++i) {
     if (gKeyConversions[i].geckoKeyCode == aKeyCode) {
       return gKeyConversions[i].charCode;
     }
@@ -1348,8 +1356,8 @@ nsresult nsCocoaUtils::GetScreenCapturePermissionState(
   // names if the calling application has been authorized to record the
   // screen. We use the window name, window level, and owning PID as
   // heuristics to determine if we have screen recording permission.
-  AutoCFRelease<CFArrayRef> windowArray =
-      CGWindowListCopyWindowInfo(kCGWindowListOptionAll, kCGNullWindowID);
+  AutoCFTypeRef<CFArrayRef> windowArray(
+      CGWindowListCopyWindowInfo(kCGWindowListOptionAll, kCGNullWindowID));
   if (!windowArray) {
     LOG("GetScreenCapturePermissionState() ERROR: got NULL window info list");
     return NS_ERROR_UNEXPECTED;
@@ -1396,6 +1404,19 @@ nsresult nsCocoaUtils::GetScreenCapturePermissionState(
       continue;
     }
 
+    // macOS versions 12.2 (Monterey) or later have a status indicator when the
+    // microphone is in use (an orange dot). This is implemented as a window
+    // owned by the window server process. The permission check logic queries
+    // window server for all windows and assumes it has the required permission
+    // if it can read any window name that is at dock or normal level.
+    // The StatusIndicator window is an exception and needs to be skipped
+    // because it is owned by window server process and therefore when querying
+    // the window server, the name is always readable.
+    if (kCFCompareEqualTo ==
+        CFStringCompare(windowName, CFSTR("StatusIndicator"), 0)) {
+      continue;
+    }
+
     CFNumberRef windowLayerRef = reinterpret_cast<CFNumberRef>(
         CFDictionaryGetValue(windowDict, kCGWindowLayer));
     int32_t windowLayer;
@@ -1531,8 +1552,8 @@ void nsCocoaUtils::ResolveAudioCapturePromises(bool aGranted) {
 //
 nsresult nsCocoaUtils::MaybeRequestScreenCapturePermission() {
   LOG("MaybeRequestScreenCapturePermission()");
-  AutoCFRelease<CGImageRef> image =
-      CGDisplayCreateImageForRect(kCGDirectMainDisplay, CGRectMake(0, 0, 1, 1));
+  AutoCFTypeRef<CGImageRef> image(CGDisplayCreateImageForRect(
+      kCGDirectMainDisplay, CGRectMake(0, 0, 1, 1)));
   return NS_OK;
 }
 
@@ -1688,25 +1709,23 @@ NSString* nsCocoaUtils::GetTitleForURLFromPasteboardItem(
   NS_OBJC_END_TRY_BLOCK_RETURN(nil);
 }
 
-void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
-    nsITransferable* aTransferable, const nsCString& aFlavor,
-    NSPasteboardItem* aItem) {
-  NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
+already_AddRefed<nsISupports> nsCocoaUtils::GetDataFromPasteboardItem(
+    const nsACString& aFlavor, NSPasteboardItem* aItem) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
-  if (!aTransferable || !aItem) {
-    return;
+  if (!aItem) {
+    return nullptr;
   }
 
   MOZ_LOG(gCocoaUtilsLog, LogLevel::Info,
-          ("nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem: looking "
-           "for pasteboard data of "
-           "type %s\n",
-           aFlavor.get()));
+          ("nsCocoaUtils::GetDataFromPasteboardItem: looking for pasteboard "
+           "data of type %s\n",
+           PromiseFlatCString(aFlavor).get()));
 
   if (aFlavor.EqualsLiteral(kFileMime)) {
     NSString* filePath = nsCocoaUtils::GetFilePathFromPasteboardItem(aItem);
     if (!filePath) {
-      return;
+      return nullptr;
     }
 
     unsigned int stringLength = [filePath length];
@@ -1714,22 +1733,21 @@ void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
         (stringLength + 1) * sizeof(char16_t);  // in bytes
     char16_t* clipboardDataPtr = (char16_t*)malloc(dataLength);
     if (!clipboardDataPtr) {
-      return;
+      return nullptr;
     }
 
     [filePath getCharacters:reinterpret_cast<unichar*>(clipboardDataPtr)];
     clipboardDataPtr[stringLength] = 0;  // null terminate
 
     nsCOMPtr<nsIFile> file;
-    nsresult rv = NS_NewLocalFile(nsDependentString(clipboardDataPtr), true,
+    nsresult rv = NS_NewLocalFile(nsDependentString(clipboardDataPtr),
                                   getter_AddRefs(file));
     free(clipboardDataPtr);
     if (NS_FAILED(rv)) {
-      return;
+      return nullptr;
     }
 
-    aTransferable->SetTransferData(aFlavor.get(), file);
-    return;
+    return file.forget();
   }
 
   if (aFlavor.EqualsLiteral(kCustomTypesMime)) {
@@ -1738,17 +1756,17 @@ void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
                                    arrayWithObject:kMozCustomTypesPboardType]];
     if (!availableType ||
         !nsCocoaUtils::IsValidPasteboardType(availableType, false)) {
-      return;
+      return nullptr;
     }
     NSData* pasteboardData = [aItem dataForType:availableType];
     if (!pasteboardData) {
-      return;
+      return nullptr;
     }
 
     unsigned int dataLength = [pasteboardData length];
     void* clipboardDataPtr = malloc(dataLength);
     if (!clipboardDataPtr) {
-      return;
+      return nullptr;
     }
     [pasteboardData getBytes:clipboardDataPtr length:dataLength];
 
@@ -1757,9 +1775,8 @@ void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
         aFlavor, clipboardDataPtr, dataLength,
         getter_AddRefs(genericDataWrapper));
 
-    aTransferable->SetTransferData(aFlavor.get(), genericDataWrapper);
     free(clipboardDataPtr);
-    return;
+    return genericDataWrapper.forget();
   }
 
   NSString* pString = nil;
@@ -1799,7 +1816,7 @@ void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
     unsigned int dataLength = [stringData length];
     void* clipboardDataPtr = malloc(dataLength);
     if (!clipboardDataPtr) {
-      return;
+      return nullptr;
     }
     [stringData getBytes:clipboardDataPtr length:dataLength];
 
@@ -1822,9 +1839,8 @@ void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
     nsPrimitiveHelpers::CreatePrimitiveForData(
         aFlavor, clipboardDataPtrNoBOM, dataLength,
         getter_AddRefs(genericDataWrapper));
-    aTransferable->SetTransferData(aFlavor.get(), genericDataWrapper);
     free(clipboardDataPtr);
-    return;
+    return genericDataWrapper.forget();
   }
 
   // We have never supported this on Mac OS X, we should someday. Normally
@@ -1838,5 +1854,28 @@ void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
   }
   */
 
+  return nullptr;
+
+  NS_OBJC_END_TRY_BLOCK_RETURN(nullptr);
+}
+
+void nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem(
+    nsITransferable* aTransferable, const nsCString& aFlavor,
+    NSPasteboardItem* aItem) {
+  NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
+
+  if (!aTransferable || !aItem) {
+    return;
+  }
+
+  MOZ_LOG(gCocoaUtilsLog, LogLevel::Info,
+          ("nsCocoaUtils::SetTransferDataForTypeFromPasteboardItem: looking "
+           "for pasteboard data of type %s\n",
+           aFlavor.get()));
+
+  if (nsCOMPtr<nsISupports> data = GetDataFromPasteboardItem(aFlavor, aItem)) {
+    aTransferable->SetTransferData(aFlavor.get(), data);
+  }
+
   NS_OBJC_END_TRY_IGNORE_BLOCK;
 }
