@@ -28,6 +28,10 @@ class CompositorWidgetParent final : public PCompositorWidgetParent,
   mozilla::ipc::IPCResult RecvNotifyClientSizeChanged(
       const LayoutDeviceIntSize& aClientSize) override;
 
+  mozilla::ipc::IPCResult RecvCleanupResources() override;
+  mozilla::ipc::IPCResult RecvSetRenderingSurface(
+      const uintptr_t& aXWindow) override;
+
  private:
   RefPtr<VsyncObserver> mVsyncObserver;
 };
