@@ -8,22 +8,25 @@
 #include <windows.h>
 #include <shellapi.h>
 #include "nsStyleConsts.h"
-#include "nsUXThemeData.h"
 #include "nsUXThemeConstants.h"
+#include "nsWindowDefs.h"
 #include "nsWindowsHelpers.h"
 #include "WinUtils.h"
 #include "WindowsUIUtils.h"
 #include "mozilla/FontPropertyTypes.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/WidgetWindowsMetrics.h"
+#include "mozilla/intl/LocaleService.h"
 #include "mozilla/widget/WinRegistry.h"
 
+#define AVG2(a, b) (((a) + (b) + 1) >> 1)
+
 using namespace mozilla;
 using namespace mozilla::widget;
 
-static Maybe<nscolor> GetColorFromTheme(nsUXThemeClass cls, int32_t aPart,
+static Maybe<nscolor> GetColorFromTheme(UXThemeClass cls, int32_t aPart,
                                         int32_t aState, int32_t aPropId) {
   COLORREF color;
-  HRESULT hr = GetThemeColor(nsUXThemeData::GetTheme(cls), aPart, aState,
+  HRESULT hr = GetThemeColor(nsLookAndFeel::GetTheme(cls), aPart, aState,
                              aPropId, &color);
   if (hr == S_OK) {
     return Some(COLOREF_2_NSRGB(color));
@@ -47,22 +50,60 @@ static int32_t GetTooltipOffsetVertical() {
                     float(cursorSize) / float(kDefaultCursorSize));
 }
 
-static bool SystemWantsDarkTheme() {
-  if (nsUXThemeData::IsHighContrastOn()) {
-    return LookAndFeel::IsDarkColor(
-        LookAndFeel::Color(StyleSystemColor::Window, ColorScheme::Light,
-                           LookAndFeel::UseStandins::No));
+UXThemeHandle::~UXThemeHandle() { Close(); }
+
+void UXThemeHandle::OpenOnce(LPCWSTR aClassList) {
+  if (mHandle.isSome()) {
+    return;
   }
 
-  WinRegistry::Key key(
-      HKEY_CURRENT_USER,
-      u"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize"_ns,
-      WinRegistry::KeyMode::QueryValue);
-  if (NS_WARN_IF(!key)) {
-    return false;
+  mHandle = Some(OpenThemeData(nullptr, aClassList));
+}
+
+void UXThemeHandle::Close() {
+  if (mHandle.isNothing()) {
+    return;
   }
-  uint32_t light = key.GetValueAsDword(u"AppsUseLightTheme"_ns).valueOr(1);
-  return !light;
+
+  if (HANDLE rawHandle = mHandle.extract()) {
+    CloseThemeData(rawHandle);
+  }
+}
+
+UXThemeHandle::operator HANDLE() { return mHandle.valueOr(nullptr); }
+
+static const wchar_t* GetUXThemeClassName(UXThemeClass aClass) {
+  switch (aClass) {
+    case UXThemeClass::Button:
+      return L"Button";
+    case UXThemeClass::Edit:
+      return L"Edit";
+    case UXThemeClass::Toolbar:
+      return L"Toolbar";
+    case UXThemeClass::Progress:
+      return L"Progress";
+    case UXThemeClass::Tab:
+      return L"Tab";
+    case UXThemeClass::Trackbar:
+      return L"Trackbar";
+    case UXThemeClass::Combobox:
+      return L"Combobox";
+    case UXThemeClass::Listview:
+      return L"Listview";
+    case UXThemeClass::Menu:
+      return L"Menu";
+    case UXThemeClass::NumClasses:
+      break;
+  }
+  MOZ_ASSERT_UNREACHABLE("unknown uxtheme class");
+  return L"";
+}
+
+HANDLE nsLookAndFeel::GetTheme(UXThemeClass aClass) {
+  auto& handle =
+      static_cast<nsLookAndFeel*>(GetInstance())->mThemeHandles[aClass];
+  handle.OpenOnce(GetUXThemeClassName(aClass));
+  return handle;
 }
 
 uint32_t nsLookAndFeel::SystemColorFilter() {
@@ -78,8 +119,10 @@ uint32_t nsLookAndFeel::SystemColorFilter() {
 }
 
 nsLookAndFeel::nsLookAndFeel() {
-  mozilla::Telemetry::Accumulate(mozilla::Telemetry::TOUCH_ENABLED_DEVICE,
-                                 WinUtils::IsTouchDeviceSupportPresent());
+  glean::widget::touch_enabled_device
+      .EnumGet(static_cast<glean::widget::TouchEnabledDeviceLabel>(
+          WinUtils::IsTouchDeviceSupportPresent()))
+      .Add();
 }
 
 nsLookAndFeel::~nsLookAndFeel() = default;
@@ -92,22 +135,21 @@ void nsLookAndFeel::RefreshImpl() {
   nsXPLookAndFeel::RefreshImpl();
 }
 
-static bool UseNonNativeMenuColors(ColorScheme aScheme) {
-  return !LookAndFeel::GetInt(LookAndFeel::IntID::UseAccessibilityTheme) ||
-         aScheme == ColorScheme::Dark;
-}
-
 nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
                                        nscolor& aColor) {
   EnsureInit();
 
+  auto UseNonNativeMenuColors = [&]() -> bool {
+    return !mHighContrastOn || aScheme == ColorScheme::Dark;
+  };
+
   auto IsHighlightColor = [&] {
     switch (aID) {
       case ColorID::MozButtonhoverface:
       case ColorID::MozButtonactivetext:
-        return nsUXThemeData::IsHighContrastOn();
+        return mHighContrastOn;
       case ColorID::MozMenuhover:
-        return !UseNonNativeMenuColors(aScheme);
+        return !UseNonNativeMenuColors();
       case ColorID::Highlight:
       case ColorID::Selecteditem:
         // We prefer the generic dark selection color if we don't have an
@@ -125,14 +167,14 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
     switch (aID) {
       case ColorID::MozButtonhovertext:
       case ColorID::MozButtonactiveface:
-        return nsUXThemeData::IsHighContrastOn();
+        return mHighContrastOn;
       case ColorID::MozMenubarhovertext:
-        if (UseNonNativeMenuColors(aScheme)) {
+        if (UseNonNativeMenuColors()) {
           return false;
         }
         [[fallthrough]];
       case ColorID::MozMenuhovertext:
-        if (UseNonNativeMenuColors(aScheme)) {
+        if (UseNonNativeMenuColors()) {
           return false;
         }
         return !mColorMenuHoverText;
@@ -167,7 +209,7 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
     return NS_OK;
   }
 
-  // Titlebar colors are color-scheme aware.
+  // Titlebar and menu colors are color-scheme aware.
   switch (aID) {
     case ColorID::Activecaption:
       aColor = mTitlebarColors.Get(aScheme, true).mBg;
@@ -187,6 +229,43 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
     case ColorID::Inactiveborder:
       aColor = mTitlebarColors.Get(aScheme, false).mBorder;
       return NS_OK;
+    case ColorID::MozMenuhover:
+      MOZ_ASSERT(UseNonNativeMenuColors());
+      if (WinUtils::MicaPopupsEnabled()) {
+        aColor = aScheme == ColorScheme::Dark ? NS_RGBA(255, 255, 255, 30)
+                                              : NS_RGBA(0, 0, 0, 30);
+      } else {
+        aColor = aScheme == ColorScheme::Dark ? *GenericDarkColor(aID)
+                                              : NS_RGB(0xe0, 0xe0, 0xe6);
+      }
+      return NS_OK;
+    case ColorID::MozMenuhoverdisabled:
+      if (UseNonNativeMenuColors()) {
+        if (WinUtils::MicaPopupsEnabled()) {
+          aColor = aScheme == ColorScheme::Dark ? NS_RGBA(255, 255, 255, 10)
+                                                : NS_RGBA(0, 0, 0, 10);
+        } else {
+          aColor = aScheme == ColorScheme::Dark ? *GenericDarkColor(aID)
+                                                : NS_RGB(0xf0, 0xf0, 0xf3);
+        }
+      } else {
+        aColor = NS_TRANSPARENT;
+      }
+      return NS_OK;
+    case ColorID::Menu: {
+      if (UseNonNativeMenuColors()) {
+        if (WinUtils::MicaPopupsEnabled()) {
+          aColor = aScheme == ColorScheme::Dark ? NS_RGBA(0, 0, 0, 153)
+                                                : NS_RGBA(255, 255, 255, 153);
+        } else {
+          aColor = aScheme == ColorScheme::Dark ? *GenericDarkColor(aID)
+                                                : NS_RGB(0xf9, 0xf9, 0xfb);
+        }
+      } else {
+        aColor = GetColorForSysColorIndex(COLOR_MENU);
+      }
+      return NS_OK;
+    }
     default:
       break;
   }
@@ -256,13 +335,13 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
       idx = COLOR_GRAYTEXT;
       break;
     case ColorID::MozMenubarhovertext:
-      if (UseNonNativeMenuColors(aScheme)) {
+      if (UseNonNativeMenuColors()) {
         aColor = kNonNativeMenuText;
         return NS_OK;
       }
       [[fallthrough]];
     case ColorID::MozMenuhovertext:
-      if (UseNonNativeMenuColors(aScheme)) {
+      if (UseNonNativeMenuColors()) {
         aColor = kNonNativeMenuText;
         return NS_OK;
       }
@@ -272,32 +351,14 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
       }
       idx = COLOR_HIGHLIGHTTEXT;
       break;
-    case ColorID::MozMenuhover:
-      MOZ_ASSERT(UseNonNativeMenuColors(aScheme));
-      aColor = NS_RGB(0xe0, 0xe0, 0xe6);
-      return NS_OK;
-    case ColorID::MozMenuhoverdisabled:
-      if (UseNonNativeMenuColors(aScheme)) {
-        aColor = NS_RGB(0xf0, 0xf0, 0xf3);
-        return NS_OK;
-      }
-      aColor = NS_TRANSPARENT;
-      return NS_OK;
     case ColorID::Infobackground:
       idx = COLOR_INFOBK;
       break;
     case ColorID::Infotext:
       idx = COLOR_INFOTEXT;
       break;
-    case ColorID::Menu:
-      if (UseNonNativeMenuColors(aScheme)) {
-        aColor = NS_RGB(0xf9, 0xf9, 0xfb);
-        return NS_OK;
-      }
-      idx = COLOR_MENU;
-      break;
     case ColorID::Menutext:
-      if (UseNonNativeMenuColors(aScheme)) {
+      if (UseNonNativeMenuColors()) {
         aColor = kNonNativeMenuText;
         return NS_OK;
       }
@@ -317,7 +378,6 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
       break;
     case ColorID::Threedlightshadow:
     case ColorID::Buttonborder:
-    case ColorID::MozDisabledfield:
     case ColorID::MozSidebarborder:
       idx = COLOR_3DLIGHT;
       break;
@@ -333,14 +393,20 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
     case ColorID::Windowtext:
       idx = COLOR_WINDOWTEXT;
       break;
-    case ColorID::MozEventreerow:
-    case ColorID::MozOddtreerow:
+    case ColorID::MozDisabledfield:
+      idx = mHighContrastOn ? COLOR_BTNFACE : COLOR_3DLIGHT;
+      break;
     case ColorID::Field:
+      idx = mHighContrastOn ? COLOR_BTNFACE : COLOR_WINDOW;
+      break;
+    case ColorID::Fieldtext:
+      idx = mHighContrastOn ? COLOR_BTNTEXT : COLOR_WINDOWTEXT;
+      break;
+    case ColorID::MozOddtreerow:
     case ColorID::MozSidebar:
     case ColorID::MozCombobox:
       idx = COLOR_WINDOW;
       break;
-    case ColorID::Fieldtext:
     case ColorID::MozSidebartext:
     case ColorID::MozComboboxtext:
       idx = COLOR_WINDOWTEXT;
@@ -364,9 +430,27 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, ColorScheme aScheme,
     case ColorID::MozColheaderactivetext:
       idx = COLOR_WINDOWTEXT;
       break;
-    case ColorID::MozNativehyperlinktext:
+    case ColorID::Visitedtext: {
+      if (mHighContrastOn) {
+        // The fallback visited link color on HCM (given there's no
+        // system-provided one) is produced by preserving the foreground's
+        // green and averaging the foreground and background for the red and
+        // blue.  This is how IE and Edge do it too.
+        auto windowText = GetColorForSysColorIndex(COLOR_WINDOWTEXT);
+        auto window = GetColorForSysColorIndex(COLOR_WINDOW);
+        aColor = NS_RGB(AVG2(NS_GET_R(windowText), NS_GET_R(window)),
+                        NS_GET_G(windowText),
+                        AVG2(NS_GET_B(windowText), NS_GET_B(window)));
+      } else {
+        // Otherwise use the stand-in.
+        aColor = GetStandinForNativeColor(aID, aScheme);
+      }
+      return NS_OK;
+    }
+    case ColorID::Linktext:
       idx = COLOR_HOTLIGHT;
       break;
+    case ColorID::Activetext:
     case ColorID::Marktext:
     case ColorID::Mark:
     case ColorID::SpellCheckerUnderline:
@@ -442,7 +526,7 @@ nsresult nsLookAndFeel::NativeGetInt(IntID aID, int32_t& aResult) {
       // High contrast is a misnomer under Win32 -- any theme can be used with
       // it, e.g. normal contrast with large fonts, low contrast, etc. The high
       // contrast flag really means -- use this theme and don't override it.
-      aResult = nsUXThemeData::IsHighContrastOn();
+      aResult = mHighContrastOn;
       break;
     case IntID::ScrollArrowStyle:
       aResult = eScrollArrowStyle_Single;
@@ -462,42 +546,20 @@ nsresult nsLookAndFeel::NativeGetInt(IntID aID, int32_t& aResult) {
     case IntID::TreeScrollLinesMax:
       aResult = 3;
       break;
-    case IntID::WindowsAccentColorInTitlebar: {
+    case IntID::WindowsAccentColorInTitlebar:
       aResult = mTitlebarColors.mUseAccent;
-    } break;
+      break;
+    case IntID::WindowsMica:
+      aResult = WinUtils::MicaEnabled();
+      break;
+    case IntID::WindowsMicaPopups:
+      aResult = WinUtils::MicaPopupsEnabled();
+      break;
     case IntID::AlertNotificationOrigin:
       aResult = 0;
-      {
-        // Get task bar window handle
-        HWND shellWindow = FindWindowW(L"Shell_TrayWnd", nullptr);
-
-        if (shellWindow != nullptr) {
-          // Determine position
-          APPBARDATA appBarData;
-          appBarData.hWnd = shellWindow;
-          appBarData.cbSize = sizeof(appBarData);
-          if (SHAppBarMessage(ABM_GETTASKBARPOS, &appBarData)) {
-            // Set alert origin as a bit field - see LookAndFeel.h
-            // 0 represents bottom right, sliding vertically.
-            switch (appBarData.uEdge) {
-              case ABE_LEFT:
-                aResult = NS_ALERT_HORIZONTAL | NS_ALERT_LEFT;
-                break;
-              case ABE_RIGHT:
-                aResult = NS_ALERT_HORIZONTAL;
-                break;
-              case ABE_TOP:
-                aResult = NS_ALERT_TOP;
-                [[fallthrough]];
-              case ABE_BOTTOM:
-                // If the task bar is right-to-left,
-                // move the origin to the left
-                if (::GetWindowLong(shellWindow, GWL_EXSTYLE) & WS_EX_LAYOUTRTL)
-                  aResult |= NS_ALERT_LEFT;
-                break;
-            }
-          }
-        }
+      if (intl::LocaleService::GetInstance()->IsAppLocaleRTL()) {
+        // If the task bar is right-to-left, move the origin to the left
+        aResult |= NS_ALERT_LEFT;
       }
       break;
     case IntID::IMERawInputUnderlineStyle:
@@ -542,9 +604,20 @@ nsresult nsLookAndFeel::NativeGetInt(IntID aID, int32_t& aResult) {
     case IntID::TooltipOffsetVertical:
       aResult = GetTooltipOffsetVertical();
       break;
-    case IntID::SystemUsesDarkTheme:
-      aResult = SystemWantsDarkTheme();
+    case IntID::SystemUsesDarkTheme: {
+      if (mHighContrastOn) {
+        aResult =
+            LookAndFeel::IsDarkColor(GetColorForSysColorIndex(COLOR_WINDOW));
+      } else {
+        WinRegistry::Key key(
+            HKEY_CURRENT_USER,
+            u"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize"_ns,
+            WinRegistry::KeyMode::QueryValue);
+        aResult =
+            key && !key.GetValueAsDword(u"AppsUseLightTheme"_ns).valueOr(1);
+      }
       break;
+    }
     case IntID::SystemScrollbarSize:
       aResult = std::max(WinUtils::GetSystemMetricsForDpi(SM_CXVSCROLL, 96),
                          WinUtils::GetSystemMetricsForDpi(SM_CXHSCROLL, 96));
@@ -590,6 +663,21 @@ nsresult nsLookAndFeel::NativeGetInt(IntID aID, int32_t& aResult) {
       aResult = enable;
       break;
     }
+    case IntID::PointingDeviceKinds: {
+      LookAndFeel::PointingDeviceKinds result =
+          LookAndFeel::PointingDeviceKinds::None;
+      if (WinUtils::SystemHasMouse()) {
+        result |= LookAndFeel::PointingDeviceKinds::Mouse;
+      }
+      if (WinUtils::SystemHasTouch()) {
+        result |= LookAndFeel::PointingDeviceKinds::Touch;
+      }
+      if (WinUtils::SystemHasPen()) {
+        result |= LookAndFeel::PointingDeviceKinds::Pen;
+      }
+      aResult = static_cast<int32_t>(result);
+      break;
+    }
     default:
       aResult = 0;
       res = NS_ERROR_FAILURE;
@@ -785,7 +873,7 @@ auto nsLookAndFeel::ComputeTitlebarColors() -> TitlebarColors {
                            GetColorForSysColorIndex(COLOR_INACTIVECAPTIONTEXT),
                            GetColorForSysColorIndex(COLOR_INACTIVEBORDER)};
 
-  if (!nsUXThemeData::IsHighContrastOn()) {
+  if (!mHighContrastOn) {
     // Use our non-native colors.
     result.mActiveLight = {
         GetStandinForNativeColor(ColorID::Activecaption, ColorScheme::Light),
@@ -828,6 +916,12 @@ auto nsLookAndFeel::ComputeTitlebarColors() -> TitlebarColors {
   result.mAccentInactive = dwmKey.GetValueAsDword(u"AccentColorInactive"_ns);
   result.mAccentInactiveText = GetAccentColorText(result.mAccentInactive);
 
+  if (WinUtils::MicaEnabled()) {
+    // Use transparent titlebar backgrounds when using mica.
+    result.mActiveDark.mBg = result.mActiveLight.mBg =
+        result.mInactiveDark.mBg = result.mInactiveLight.mBg = NS_TRANSPARENT;
+  }
+
   // The ColorPrevalence value is set to 1 when the "Show color on title bar"
   // setting in the Color section of Window's Personalization settings is
   // turned on.
@@ -871,14 +965,37 @@ auto nsLookAndFeel::ComputeTitlebarColors() -> TitlebarColors {
   return result;
 }
 
+nsresult nsLookAndFeel::GetKeyboardLayoutImpl(nsACString& aLayout) {
+  char layout[KL_NAMELENGTH];
+  if (!::GetKeyboardLayoutNameA(layout)) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  aLayout.Assign(layout);
+
+  return NS_OK;
+}
+
 void nsLookAndFeel::EnsureInit() {
   if (mInitialized) {
     return;
   }
   mInitialized = true;
 
-  mColorMenuHoverText =
-      ::GetColorFromTheme(eUXMenu, MENU_POPUPITEM, MPI_HOT, TMT_TEXTCOLOR);
+  for (auto& handle : mThemeHandles) {
+    handle.Close();
+  }
+
+  mHighContrastOn = []() {
+    HIGHCONTRAST hc;
+    hc.cbSize = sizeof(HIGHCONTRAST);
+    return ::SystemParametersInfo(SPI_GETHIGHCONTRAST, 0, &hc, 0) &&
+           hc.dwFlags & HCF_HIGHCONTRASTON;
+  }();
+
+  const bool neededMicaWorkaround = NeedsMicaWorkaround();
+
+  mColorMenuHoverText = ::GetColorFromTheme(UXThemeClass::Menu, MENU_POPUPITEM,
+                                            MPI_HOT, TMT_TEXTCOLOR);
 
   // Fill out the sys color table.
   for (int i = SYS_COLOR_MIN; i <= SYS_COLOR_MAX; ++i) {
@@ -929,4 +1046,10 @@ void nsLookAndFeel::EnsureInit() {
   mCurrentColorFilter = SystemColorFilter();
 
   RecordTelemetry();
+
+  if (neededMicaWorkaround != NeedsMicaWorkaround()) {
+    WinUtils::UpdateMicaInAllWindows();
+  }
 }
+
+#undef AVG2
